class AssetsController < ApplicationController  
  #helper :libra
  include LibraHelper
  
  before_filter :content_type_check, :only => :new
  before_filter :verify_editor_permissions, :only => [:update, :destroy]
  
  # Uses the update_indexed_attributes method provided by ActiveFedora::Base
  # This should behave pretty much like the ActiveRecord update_indexed_attributes method
  # For more information, see the ActiveFedora docs.
  # 
  # Examples
  # put :update, :id=>"_PID_", "document"=>{"subject"=>{"-1"=>"My Topic"}}
  # Appends a new "subject" value of "My Topic" to any appropriate datasreams in the _PID_ document.
  # put :update, :id=>"_PID_", "document"=>{"medium"=>{"1"=>"Paper Document", "2"=>"Image"}}
  # Sets the 1st and 2nd "medium" values on any appropriate datasreams in the _PID_ document, overwriting any existing values.
  def update    
    
    af_model = retrieve_af_model(params[:content_type])
    unless af_model 
      af_model = HydrangeaArticle
    end
    @document = af_model.find(params[:id])
    
    # if an item is released, don't allow a non-superuser to update the computing id of the first person
    computing_id_params = @document.fetch_field_from_params("computing_id", params)
    if !computing_id_params.empty? and computing_id_params[0] == 0 # the first param indicates which person
      if @document.is_released? && !current_user.is_being_superuser?(session)
        flash[:error] = "You may not change the computing id on a released work."
        redirect_to(edit_catalog_url) and return
      end 
    end

    updater_method_args = prep_updater_method_args(params)

    @document.update_from_computing_id(params)
    @document.issn_lookup(params)
    @document.update_from_embargo_selection(params)
  
    logger.debug("########## attributes submitted: #{updater_method_args.inspect}")
    # this will only work if there is only one datastream being updated.
    # once ActiveFedora::MetadataDatastream supports .update_datastream_attributes, use that method instead (will also be able to pass through params["asset"] as-is without usin prep_updater_method_args!)
    if updater_method_args[:params].keys.include? [:embargo, :embargo_release_date]
      em_date = updater_method_args[:params][[:embargo, :embargo_release_date]]["0"]
      unless em_date.blank?
        begin 
          updater_method_args[:params][[:embargo, :embargo_release_date]]["0"] = Date.parse(em_date).strftime("%F")
        rescue
          flash[:error] = "Release Date is not a valid date!"
          updater_method_args[:params][[:embargo,:embargo_release_date]]["0"] = ""
          return
        end
      end
    end  
  
    if updater_method_args[:params].keys.include? [:origin_info, :date_created]
      created_date = updater_method_args[:params][[:origin_info, :date_created]]["0"]
      unless created_date.blank?
        begin
          updater_method_args[:params][[:origin_info, :date_created]]["0"] = Date.parse(created_date).strftime("%F")
        rescue
          flash[:error] = "Date Files/Data Package Created is not a valid date!"
          updater_method_args[:params][[:origin_info, :date_created]]["0"]= ""
          return
        end
      end
    end
    
    if updater_method_args[:params].keys.include? [:data_collection_start_date]
      start_date = updater_method_args[:params][[:data_collection_start_date]]["0"]
      unless start_date.blank?
        begin
          updater_method_args[:params][[:data_collection_start_date]]["0"] = Date.parse(start_date).strftime("%F")
        rescue
          flash[:error] = "Data Collection Start Date is not a valid date!"
          updater_method_args[:params][[:data_collection_start_date]]["0"] = ""
          return
        end
      end
    end
    
    if updater_method_args[:params].keys.include? [:data_collection_end_date]
      end_date = updater_method_args[:params][[:data_collection_end_date]]["0"]
      unless end_date.blank?
        begin
          updater_method_args[:params][[:data_collection_end_date]]["0"] = Date.parse(end_date).strftime("%F")
        rescue
          flash[:error] = "Data Collection End Date is not a valid date!"
          updater_method_args[:params][[:data_collection_end_date]]["0"] = ""
          return
        end
      end
    end
    
    #when you select embargo months, instead of sisMetadata, updating the rightMetadata datastream which is expected by activeFedora menthod update_indexed_attributes 
    if updater_method_args[:opts][:datastreams] == "sisMetadata"
      selection = params[:asset][:sisMetadata][":embargo_selection"]["0"]
      selection == "0" ? val = Date.today.strftime('%Y-%m-%d') : val = (Date.today + selection.to_i.months).strftime('%Y-%m-%d')
      updater_method_args[:params] = {:embargo_release_date =>val}
      updater_method_args[:opts] = {:datastreams=>"rightsMetadata"}     
    end
    
    result = @document.update_indexed_attributes(updater_method_args[:params], updater_method_args[:opts])  
   
    @document.save

    #response = attrs.keys.map{|x| escape_keys({x=>attrs[x].values})}    
    response = Hash["updated"=>[]]
    last_result_value = ""
    if result and !result.nil? 
      result.each_pair do |field_name,changed_values|
        changed_values.each_pair do |index,value|
          response["updated"] << {"field_name"=>field_name,"index"=>index,"value"=>value} 
          last_result_value = value
        end
      end 
      logger.debug("###########returning #{response.inspect}")
    end
      
    
    # If handling submission from jeditable (which will only submit one value at a time), return the value it submitted
    if params.has_key?(:field_id)
      response = last_result_value
    end
    logger.debug("###########returning #{@document.class}")
    respond_to do |want| 
      want.html {
        next_page = edit_catalog_url
        if [HydrangeaArticle, HydrangeaArticlePreprint, HydrangeaBook, HydrangeaBookChapter, HydrangeaBookPart, HydrangeaConferencePaper, 
          HydrangeaDoctoralDissertation, HydrangeaFourthYearThesis, HydrangeaMastersThesis, HydrangeaDataset].include?(@document.class)
          display_release_status_notice(@document)
          # Go on to the ETD survey if this was a successful ETD submission.
          next_page = survey_url if flash[:error].blank? && [HydrangeaDoctoralDissertation, HydrangeaFourthYearThesis, HydrangeaMastersThesis].include?(@document.class)
        end
        redirect_to next_page
      }
      want.js   { render :json=> response }
      want.textile {
        if response.kind_of?(Hash)
          response = response.values.first
        end
        render :text=> white_list( RedCloth.new(response, [:sanitize_html]).to_html )
      }
    end
  end

  def destroy
    af = ActiveFedora::Base.load_instance_from_solr(params[:id])
    the_model = ActiveFedora::ContentModel.known_models_for( af ).first
    unless the_model.nil?
      af = the_model.load_instance_from_solr(params[:id])
      assets = af.destroy_child_assets
    end
    
    is_released = af.datastreams["properties"].released_values[0] == "true" rescue false
    
    if current_user.is_being_superuser?(session) or !is_released
      af.delete
      msg = "Deleted #{params[:id]}"
      msg.concat(" and associated file_asset(s): #{assets.join(", ")}") unless assets.empty?
      flash[:notice]= msg
      redirect_to url_for(:action => 'index', :controller => "catalog", :q => nil , :f => nil)
    end
  end
  
  def content_type_check
    if ['hydrangea_fourth_year_thesis', 'hydrangea_masters_thesis', 'hydrangea_doctoral_dissertation'].include?(params[:content_type])
        unless current_user.can_upload_etd_type?(params[:content_type])
        flash[:error] = "You don't have sufficient permissions to upload that type of document."
        redirect_to catalog_index_path
      end
    end
  end
  
  def verify_editor_permissions
    @response, @document = get_solr_response_for_doc_id(params[:id])
    unless document_editor? or current_user.can_be_superuser?
      Rails.logger.info("user #{current_user.inspect} does not have permission to edit #{params[:id]}")
      flash[:error] = "You don't have sufficient permission to modify this document."
      redirect_to catalog_index_path
    end
    @response = nil
    @document = nil
  end
end
