class CatalogController
  
  before_filter :resolve_sort, :only => :index
  
  # displays values and pagination links for a single facet field
   def facet
     @extra_controller_params ||= {}
     @extra_controller_params.merge!(:q=>build_lucene_query(params[:q]))
     # adding the following for facet_pagination with Lucene queries to avoide NPE
     params[:qt] = "dismax"
     @pagination = get_facet_pagination(params[:id], @extra_controller_params)
   end
  
  
  # overriding so that viewing context is not stored in session
  def edit
    params[:viewing_context] = "edit"
    render :action=>:show
  end
  
  def new
  end
  
  # overriding so that viewing context is not stored in session
  def show
    if params[:viewing_context] == "edit"
      if (editor? or current_user.can_be_superuser?)
        if params[:files]
          redirect_to(:action=>:edit, :files=>true) and return
        else
          redirect_to(:action=>:edit) and return
        end
      else
        # the person isn't an editor, switch them back to browse
        params[:viewing_context] = "browse"
      end
    end
    params = {:qt=>"dismax",:q=>"*:*",:rows=>UVA_SOLR_ROWS_NUM,:facet=>"true", :facets=>{:fields=>Blacklight.config[:facet][:field_names]}}
    @facet_lookup = Blacklight.solr.find params
    respond_to do |format|
      format.html {}
      format.json { render :json => Hash.from_xml(@document_fedora.datastreams_in_memory['descMetadata'].to_xml).to_json }
      format.xml { render :xml => @document_fedora.datastreams_in_memory["descMetadata"].to_xml }
    end
  end
  
  
  def default_html_head
  
    javascript_includes << ['http://ajax.googleapis.com/ajax/libs/jquery/1.4.2/jquery.min.js', 'http://ajax.googleapis.com/ajax/libs/jqueryui/1.8.1/jquery-ui.min.js', 'tiptip/jquery.tipTip.minified.js']
    javascript_includes << ['application']
    
    javascript_includes << ['blacklight', 'application', { :plugin=>:blacklight } ]
    
    stylesheet_links << ['yui', 'jquery/ui-lightness/jquery-ui-1.8.1.custom.css', 'application', {:plugin=>:blacklight, :media=>'all'}]
    stylesheet_links << ['/javascripts/tiptip/tipTip.css', 'redmond/jquery-ui-1.8.5.custom.css', {:media=>'all'}]      
    stylesheet_links << ['styles', 'hydrangea', {:media=>'all'}]
  end 
  
  def about
  end
  
  def licenses
  end
  
  def terms
  end  
  
  def ldap_photo
    person = Ldap::Person.new(params[:id])
    send_data person.photo, :type => "image/jpeg", :file_name => person.computing_id, :disposition => 'inline' unless !person.has_photo?
    return
  end

  # Show ETD survey (catalog/show_survey.html.erb) if a URL is provided.
  # This is not applocable now Jan.2016
  def show_survey
    # render(:action => :edit) unless ETD_SURVEY_URL
  end

  protected
  
  def resolve_sort
    # if there is a query, but no sort set sort by relevancy
    if params[:q] and !params[:q].blank? and params[:sort_key].blank?
      solr_sort = solr_sort_val('relevancy')
    else
      # sort by the requested sort, if invalid, sort by deposited_d
      solr_sort = solr_sort_val(params[:sort_key]) || solr_sort_val('year_d')
    end
    params[:sort] = solr_sort
  end
  
  # lookup the solr sort from the config for this sort_key
  def solr_sort_val(sort_key)
    return if sort_key.blank?
    Blacklight.config[:sort_fields][sort_key][1] rescue return
  end
    
  #  overriding so that superuer can edit it. --ensures that the document is editable
  def check_edit_permission
    unless editor? or current_user.can_be_superuser?
      session[:viewing_context] = "browse"
      flash[:notice] = "You do not have sufficient privileges to edit this document. You have been redirected to the read-only view."
      redirect_to :action=>:show
    end
  end
end
