class DepartmentFacetMappingsController < ApplicationController

  include MediaShelf::ActiveFedoraHelper
  before_filter :require_solr, :require_fedora

  include Blacklight::SolrHelper

  before_filter :verify_department_facet_mappings_user
  
  # list all the department facet mappings
  def index
    @department_facet_mappings = DepartmentFacetMapping.find(:all)
  end
  
  # form for creating a new department facet mapping
  def new
    @department_facet_mapping = DepartmentFacetMapping.new
  end
  
  # create the department facet mapping
  def create
    @department_facet_mapping = DepartmentFacetMapping.new(params[:department_facet_mapping])
    respond_to do |format|
      if @department_facet_mapping.save
        resolrize_mapping(@department_facet_mapping)
        flash[:notice] = 'Facet mapping successfully created'
        format.html { redirect_to(department_facet_mappings_path) }
      else
        format.html { render :action => 'new' }
      end
    end
  end
  
  # delete a department facet mapping
  def destroy
    @department_facet_mapping = DepartmentFacetMapping.find(params[:id])
    @department_facet_mapping.destroy
    resolrize_mapping(@department_facet_mapping)
    respond_to do |format|
      format.html { redirect_to(department_facet_mappings_path) }
    end
  end
  
  protected
  
  def resolrize_mapping(department_facet_mapping)
    pids = affected_pids(department_facet_mapping)
    pids.each do |pid|
      Solrizer::Fedora::Solrizer.new.solrize(pid)
    end
  end
  
  def affected_pids(department_facet_mapping)
    opts = {
      :f => {
        :uva_department_facet => [department_facet_mapping.department_facet]
      },
      :qt => "dismax",
      :per_page => 100
    }
    response, documents = get_search_results(opts)
    documents.collect { |doc| doc[:id] }
  end
  
  def verify_department_facet_mappings_user
    if current_user.nil?
      flash[:error] = "You must be logged in to manage department facet mappings."
      redirect_to root_path
    elsif !current_user.can_be_superuser?
      flash[:error] = "You must be a superuser to manage department facet mappings."
      redirect_to root_path
    end
  end

end