require_dependency "vendor/plugins/hydra_repository/app/controllers/file_assets_controller.rb"

class FileAssetsController 
  
  include LibraHelper
  
  before_filter :set_document, :only=>[:create]
  before_filter :verify_file, :only=>[:create]
  before_filter :set_filename, :only=>[:create]
  before_filter :verify_mime_type, :only=>[:create]
  before_filter :verify_file_size, :only=>[:create]
  
  # this needs to be done first, so that if an error occurs, we can still render the list of files already in place
  def set_document
    if !params[:container_id].nil?
      @container =  ActiveFedora::Base.load_instance(params[:container_id])
      @response, @document = get_solr_response_for_doc_id(params[:container_id])
      @solr_result = @container.file_objects({:response_format=>:solr, :rows=>UVA_SOLR_ROWS_NUM})
    end
  end
  
  # check to make sure there actually is a file
  def set_filename
    params[:Filename] = params[:Filedata].original_filename rescue()
  end
  
  # for most data types, PDFs are the only accepted format
  # check if this is the case, then complain if the file isn't a PDF
  def verify_mime_type    
    the_model = get_model(params[:container_id])
    return if the_model.respond_to?(:only_pdfs?) and !the_model.only_pdfs?
    
   #allow mime type of file for supplement file in etd 
    unless ( mime_type(params[:Filename]) == "application/pdf" ||  params[:file_type_select] == "supplement")
      @error = "Only PDFs will be accepted for this type of work."
      render :index, :layout => 'file_error'
    end
  end
  
  def verify_file
    if params[:Filedata].nil?
      @error = "You must select a file to upload"
      render :index, :layout => 'file_error' and return
    end
  end
  
  def verify_file_size
    unless current_user.is_being_superuser?(session)
      return unless params[:Filedata].size > 100.megabytes
      @error = "File size is limited to 100 MB."
      render :index, :layout => 'file_error'
    end
  end

  def index
    if params[:layout] == "false"
      # action = "index_embedded"
      layout = false
    end
    if !params[:container_id].nil?
      @response, @document = get_solr_response_for_doc_id(params[:container_id])
      @container =  ActiveFedora::Base.load_instance(params[:container_id])
      @solr_result = @container.file_objects({:response_format=>:solr, :rows=>UVA_SOLR_ROWS_NUM})
    else
      # @solr_result = ActiveFedora::SolrService.instance.conn.query('has_model_field:info\:fedora/afmodel\:FileAsset', @search_params)
      @solr_result = FileAsset.find_by_solr(:all, {:rows=>UVA_SOLR_ROWS_NUM})
      puts @solr_result.inspect
    end
    render :action=>params[:action], :layout=>layout
  end

  def create
    @file_asset = create_and_save_file_asset_from_params
     
    unless @file_asset
      @error = "File was not uploaded"
      render :index, :layout => 'file_error' and return
    end
          
    apply_depositor_metadata(@file_asset)

    if !params[:container_id].nil?
      @container.file_objects_append(@file_asset)
      @container.save
      @response, @document = get_solr_response_for_doc_id(params[:container_id], {:rows=>UVA_SOLR_ROWS_NUM})
      @solr_result = @container.file_objects({:response_format=>:solr, :rows=>UVA_SOLR_ROWS_NUM})
    end

    ## FOR CAPTURING ANY FILE METADATA // THERE'S PROBABY A BETTER PLACE FOR THIS. 
    unless params[:asset].nil?
      updater_method_args = prep_updater_method_args(params)
      logger.debug("attributes submitted: #{updater_method_args.inspect}")
      result = @file_asset.update_indexed_attributes(updater_method_args[:params], updater_method_args[:opts])
      @file_asset.save
    end
    ##

    logger.debug "Created #{@file_asset.pid}."
    @success = true
    render :index, :layout => false
  end
   
  # Common destroy method for all AssetsControllers 
  def destroy
    # The correct implementation, with garbage collection:
    # if params.has_key?(:container_id)
    #   container = ActiveFedora::Base.load_instance(params[:container_id]) 
    #   container.file_objects_remove(params[:id])
    #   FileAsset.garbage_collect(params[:id])
    # else

    # The dirty implementation (leaves relationship in container object, deletes regardless of whether the file object has other containers)
    ActiveFedora::Base.load_instance(params[:id]).delete 
    flash[:notice]= "Deleted file"
    redirect_to url_for(:action => 'edit', :controller => "catalog", :id => params[:container_id])
  end

  
  def show
    @file_asset = FileAsset.find(params[:id])
    if (@file_asset.nil?)
      logger.warn("No such file asset: " + params[:id])
      flash[:notice]= "No such file asset."
      redirect_to(:action => 'index', :q => nil , :f => nil)
    else
      # get array of parent (container) objects for this FileAsset
      @id_array = @file_asset.containers(:response_format => :id_array, :rows=>UVA_SOLR_ROWS_NUM)
      @downloadable = false
      # A FileAsset is downloadable iff the user has read or higher access to a parent
      @id_array.each do |pid|
        @response, @document = get_solr_response_for_doc_id(pid, {:rows=>UVA_SOLR_ROWS_NUM})
        if document_reader?
          @downloadable = true
          break
        end
      end

      #If file assest is downloadable and it is pdf, create a cover for pdf dynamically
      # and combine the pdf with the cover, then replace the pdf download link with the new name
      if @downloadable
        if @file_asset.datastreams_in_memory.include?("DS1")
          if @file_asset.datastreams_in_memory["DS1"]["mimeType"] == "application/pdf"
            pdf_file = @file_asset.datastreams_in_memory["DS1"]
            cover = create_pdf_cover(pdf_file)
            pdf_filename= pdf_file.label
            pdf = "#{RAILS_ROOT}/tmp/pdf_covers/#{pdf_filename}"
            f = File.open(pdf, "wb+")
            f.write pdf_file.content
            f.close
            combined_file = pdf_combine(cover, pdf, pdf_new_name)
            # The following line is used for create combine pdf file with cover using Prawn only
            # Please keep it until we upgrade to ruby 1.9.3
            #combined_file = prawn_combine(cover, pdf, pdf_new_name)
            send_file combined_file, :type=>"application/pdf"
            File.delete(cover)
            File.delete(pdf)
          else
            send_datastream @file_asset.datastreams_in_memory["DS1"]
         end
        end
      else
        flash[:notice]= "You do not have sufficient access privileges to download this document, which has been marked private."
        redirect_to(:action => 'index', :q => nil , :f => nil)
      end
    end
  end
  
  def download_dataset_readme_template
      send_file "#{RAILS_ROOT}/public/dataset_readme_template.txt", :type=>"application/text" 
  end

  protected
  
  def get_model(pid)
    the_model = nil
    af_base = ActiveFedora::Base.load_instance(pid) rescue nil
    return the_model if af_base.nil?
    the_model = ActiveFedora::ContentModel.known_models_for( af_base ).first
    if the_model.nil?
      the_model = DcDocument
    end
    the_model
  end

end
