module HydraAccessControlsHelper
  require 'ipaddr'
  require_dependency 'vendor/plugins/hydra_repository/app/helpers/hydra_access_controls_helper.rb'
  def test_permission(permission_type)    
    # if !current_user.nil?
      if (@document == nil)
        logger.warn("SolrDocument is nil")
      end

      if current_user.nil? 
        user = "public"
        logger.debug("current_user is nil, assigning public")
      else
        user = current_user.login
      end
      
      user_groups = RoleMapper.roles(user)
      # everyone is automatically a member of the group 'public'
      user_groups.push 'public' unless user_groups.include?('public')
      # everyone is automatically a member of the group 'uetd'
      user_groups.push 'uetd' unless user_groups.include?('uetd')      
      # everyone is automatically a member of the group 'public_dataset'
      user_groups.push 'public_dataset' unless user_groups.include?('public_dataset')            
      # logged-in users are automatically members of the group "registered
      user_groups.push 'registered' unless user == "public"
      # logged-in users are automatically members of the group "uva"
      user_groups.push 'uva' unless user == "public"
      
      logger.debug("User #{user} is a member of groups: #{user_groups.inspect}")
      case permission_type
        when :edit
          logger.debug("Checking edit permissions for user: #{user}")
          group_intersection = user_groups & edit_groups
          result = !group_intersection.empty? || edit_persons.include?(user)
        when :read
          logger.debug("Checking read permissions for user: #{user}")
          group_intersection = user_groups & read_groups
          result = !group_intersection.empty? || read_persons.include?(user)
        else
          result = false
      end
      logger.debug("test_permission result: #{result}")
      return result
    # else
    #   logger.debug("nil user, test_permission returning false")
    #   return false
    # end
  end
  
  #ETDs from ASG can be accessed by uva logins besides ETD creator and superusers  
  def embargoed?
    return false if @document["embargo_release_date_dt"].blank?
    return false if current_user and current_user.is_being_superuser?(session)
    return false if current_user && current_user.login == @document["depositor_t"].first
    return true if during_embargo? and !is_ASG?(@document)
    return true if embargoed_ASG_etd_access_by_non_uva?
    return false
  end
  
  def restricted_to_uva?
    return false if document_reader?
    return false if embargoed?
    return true if @document["release_to_t"].first == "uva"  
    return false
  end
  
  def document_reader?
    return true if current_user and current_user.is_being_superuser?(session)
    return true if embargoed_ASG_etd_access_by_uva?
    return true if test_permission(:read) and !embargoed?
    return false
  end
  
  def document_editor?
    return true if current_user and current_user.is_being_superuser?(session)
    return true if test_permission(:edit)
    return false
  end
  
  def embargoed_ASG_etd_access_by_non_uva?
    return true if is_ASG?(@document) and during_embargo? and !current_user and !allowed_remote_access?
    return false
  end
  
  def embargoed_ASG_etd_access_by_uva?
    return true if is_ASG?(@document) and during_embargo? and (current_user or allowed_remote_access?)
     return false
  end
  def during_embargo?
    embargo_date = Date.parse(@document["embargo_release_date_dt"].split(/T/)[0]) unless @document["embargo_release_date_dt"].nil?
    return true if !embargo_date.nil?  && embargo_date > Date.parse(Time.now.to_s)
    return false
  end 

  #
  def allowed_remote_access?
    Rails.logger.info("~~~~~~~~request.remote_ip=#{request.remote_ip}")
    return true if uva_ip_blocks.any?{ |block| block.include?(request.remote_ip) }
    return false
  end
  
  def uva_ip_blocks
    uva_ip_ranges_list =[]
    File.open(uva_ip_range_list_file,"r").each_line do |line|
      uva_ip_ranges_list.push line
    end
    uva_ip_ranges_list.map { |subnet| IPAddr.new subnet }
  end
  
  def uva_ip_range_list_file
      UVA_IP_RANGES_LIST_DIR + "/uva_ip_list"
  end

  # This method should return proper messages for materials either embargoed or not
  # and decide whether the files should be displayed
  def messages_and_files
    file_display = false
    msg = ""
    if !document_reader?
      msg= show_message(embargo_node)
    else
      file_display = true
      msg= show_message(embargo_node)
    end
    return msg,file_display
  end
end

def embargo_node
  @document_fedora.datastreams['rightsMetadata'].find_by_terms(:embargo, :embargo_release_date) unless @document_fedora.datastreams['rightsMetadata'].nil?
end

def show_message(embargo_node)
  message = ''
  message << choose_embargo_message(embargo_node) if (!@document["embargo_release_date_dt"].blank? && during_embargo? )
  if !@document['release_to_t'].nil? && @document['release_to_t'].first == 'uva'
    message << '<br />' if (message.length > 0)
    message << restricted_message
  end
  message
end


def choose_embargo_message(embargo_node)
  if is_ASG?(@document)
    message = asg_embargo_message(embargo_node)
  else
    message = general_embargo_message(embargo_node)
  end
  message
end

def asg_embargo_message(embargo_node)
  "This item is restricted to UVa until #{embargo_node}."
end

def general_embargo_message(embargo_node)
  "This item is under embargo until #{embargo_node}."
end

def restricted_message
  "License restrictions require UVa login for this item."
end
