module HydraFedoraMetadataHelper
  require_dependency 'vendor/plugins/hydra_repository/app/helpers/hydra_fedora_metadata_helper.rb'

  def fedora_multi_field(resource, datastream_name, field_key, opts={})
    field_name = field_name_for(field_key)
    field_values = get_values_from_datastream(resource, datastream_name, field_key, opts)
    all_values = field_values.join(", ")
    base_id = generate_base_id(field_name, all_values, field_values, opts)

    name = "asset[#{datastream_name}][#{field_name}][multi]"

    body = ""
    body << "<span class=\"editable-container field\" id=\"#{base_id}-container\">"

    body << "<input class=\"editable-edit edit\" id=\"#{base_id}\" data-datastream-name=\"#{datastream_name}\" rel=\"#{field_name}\" name=\"#{name}\"  />"
    body << "<a class=\"multi-load btn\" id=\"keyword_load\">Add Keywords</a>"
    body << "<ol>"
    field_values.each_with_index do |current_value, z|
      delete_name = "asset[#{datastream_name}][#{field_name}][#{z}]"
      
      body << "<li class=\"editable-container field\" id=\"#{base_id}_#{z}-container\">"
      body << "<input class=\"editable-edit edit\" id=\"#{base_id}\" data-datastream-name=\"#{datastream_name}\" rel=\"#{field_name}\" name=\"#{delete_name}\" value=\"#{h(current_value.lstrip)}\" type=\"hidden\" />"
      
      body << "#{h(current_value)}<a href=\"\" title=\"Delete '#{h(current_value)}'\" class=\"destructive field\">Delete</a>" unless current_value.empty?
      body << "</li>"    
    end
    body << "</ol></span>"
    result = field_selectors_for(datastream_name, field_key)
    result << body
    result
  end


  def fedora_text_field(resource, datastream_name, field_key, opts={})    
    field_name = field_name_for(field_key)
    field_values = get_values_from_datastream(resource, datastream_name, field_key, opts)
    field_values = [""] if field_values.empty?
    if opts.fetch(:multiple, true)
      container_tag_type = :li
    else
      if opts.fetch(:index, false)
        field_values = [field_values[opts.fetch(:index)]]
      else
        field_values = [field_values.first]
      end
      container_tag_type = :span
    end

    computing_id = get_values_from_datastream(resource, 'descMetadata', [{:person=>0}, :computing_id], opts)
    if opts.fetch(:enabled, false) or (current_user and !current_user.can_be_superuser? and resource.respond_to?(:non_editable_fields) and resource.non_editable_fields(computing_id).include?(field_key))
      disabled_field = "disabled=\"disabled\""
    else
      disabled_field = ""
    end
    
    if opts.fetch(:label, false)
      label = opts.fetch(:label)
    else
      label = ""
    end
    
    if opts.fetch(:help, false)
      help = opts.fetch(:help)
    else
      help = ""
    end
            
    body = ""
    
    field_values.each_with_index do |current_value, z|
      base_id = generate_base_id(field_name, current_value, field_values, opts)
      if opts.fetch(:index, false)
        name = "asset[#{datastream_name}][#{field_name}][#{opts.fetch(:index)}]"
      else
        name = "asset[#{datastream_name}][#{field_name}][#{z}]"
      end
      body << "<#{container_tag_type.to_s} class=\"editable-container field\" id=\"#{base_id}-container\">"
      body << "<label for='#{base_id}'>#{label}</label>" unless label.empty?
      body << "<span class='field_help'>#{help}</span>" unless help.empty?
      body << "<a href=\"\" title=\"Delete '#{h(current_value)}'\" class=\"destructive field\">Delete</a>" if opts.fetch(:multiple, true) && !current_value.empty?
      body << "<input class=\"editable-edit edit\" id=\"#{base_id}\" data-datastream-name=\"#{datastream_name}\" rel=\"#{field_name}\" name=\"#{name}\" value=\"#{h(current_value.lstrip)}\" #{disabled_field} />"
      body << "</#{container_tag_type}>"
    end
    result = field_selectors_for(datastream_name, field_key)
    if opts.fetch(:multiple, true)
      result << content_tag(:ol, body, :rel=>field_name)
    else
      result << body
    end
    
    return result
  end
    
  def fedora_text_area(resource, datastream_name, field_key, opts={})
    fedora_textile_text_area(resource, datastream_name, field_key, opts)
  end
  
  # Textile textarea varies from the other methods in a few ways
  # Since we're using jeditable with this instead of fluid, we need to provide slightly different hooks for the javascript
  # * we are storing the datastream name in data-datastream-name so that we can construct a load url on the fly when initializing the textarea
  def fedora_textile_text_area(resource, datastream_name, field_key, opts={})
    field_name = field_name_for(field_key)
    field_values = get_values_from_datastream(resource, datastream_name, field_key, opts)
    field_values = [""] if field_values.empty?
    if opts.fetch(:multiple, true)
      container_tag_type = :li
    else
      field_values = [field_values.first]
      container_tag_type = :span
    end
    
    if opts.fetch(:label, false)
      label = opts.fetch(:label)
    else
      label = ""
    end
          
    body = ""
    
    field_values.each_with_index do |current_value, z|
      base_id = generate_base_id(field_name, current_value, field_values, opts)
      name = "asset[#{datastream_name}][#{field_name}][#{z}]"
      processed_field_value = white_list( RedCloth.new(current_value, [:sanitize_html]).to_html)
      
      body << "<#{container_tag_type.to_s} class=\"editable-container field\" id=\"#{base_id}-container\">"
      body << "<label for='#{base_id}'>#{label}</label>" unless label.empty?
      # Not sure why there is we're not allowing the for the first textile to be deleted, but this was in the original helper.
      body << "<a href=\"\" title=\"Delete '#{h(current_value)}'\" class=\"destructive field\">Delete</a>" unless z == 0
      body << "<textarea class=\"editable-edit edit\" id=\"#{base_id}\" data-datastream-name=\"#{datastream_name}\" rel=\"#{field_name}\" name=\"#{name}\">#{h(current_value)}</textarea>"
      body << "</#{container_tag_type}>"
    end
    
    result = field_selectors_for(datastream_name, field_key)
    
    if opts.fetch(:multiple, true)
      result << content_tag(:ol, body, :rel=>field_name)
    else
      result << body
    end
    return result
    
  end
  
  def fedora_checkbox(resource, datastream_name, field_key, opts={})
    result = ""
    field_values = get_values_from_datastream(resource, datastream_name, field_key, opts)
    h_name = OM::XML::Terminology.term_hierarchical_name(*field_key)    
    
    v_name = field_key.last.to_s
    
    result = field_selectors_for(datastream_name, field_key)
    
    # adding so that customized checked and unchecked values can be passed in
    checked_value = (opts[:default_values] && opts[:default_values][:checked]) ? opts[:default_values][:checked] : "yes"
    unchecked_value = (opts[:default_values] && opts[:default_values][:unchecked]) ? opts[:default_values][:unchecked] : "no"
        
    result << tag(:input, :type=>"hidden", :id=>"#{h_name}_checked_value", :value=>checked_value )
    result << tag(:input, :type=>"hidden", :id=>"#{h_name}_unchecked_value", :value=>unchecked_value )
    
    if  !field_values.first.nil? && field_values.first.downcase == checked_value.downcase
      result << tag(:input, :type=>"checkbox", :id=>h_name, :class=>"fedora-checkbox", :rel=>h_name, :name=>"asset[#{datastream_name}][#{h_name}][0]", :value=>checked_value, :checked=>"checked")
    else
      result << tag(:input, :type=>"checkbox", :id=>h_name, :class=>"fedora-checkbox", :rel=>h_name, :name=>"asset[#{datastream_name}][#{h_name}][0]", :value=>unchecked_value)
    end
    return result
  end
  
  # Expects :choices option. 
  # :choices should be a hash with value/label pairs
  # :choices => {"first_choice"=>"Apple", "second_choice"=>"Pear" }
  # If no :choices option is provided, returns a regular fedora_text_field
  def fedora_radio_button(resource, datastream_name, field_key, opts={})
    if opts[:choices].nil?
      result = fedora_text_field(resource, datastream_name, field_key, opts)
    else
      choices = opts[:choices]
      
      field_name = field_name_for(field_key)
      field_values = get_values_from_datastream(resource, datastream_name, field_key, opts)
      h_name = OM::XML::Terminology.term_hierarchical_name(*field_key)    
      default_value = opts.keys.include?(:default_value) ? opts[:default_value] : ""
      
      selected_value = field_values.empty? ? "" : field_values.first
      selected_value = default_value if selected_value.blank?

      body = ""
      z = 0
      base_id = generate_base_id(field_name, field_values.first, field_values, opts.merge({:multiple=>false}))
      name = "asset[#{datastream_name}][#{field_name}][#{z}]"
      
      result = field_selectors_for(datastream_name, field_key)
      choices.sort.each do |choice,label|
        if choice == selected_value
          result << tag(:input, :type=>"radio", :id=>"availability_#{choice}", :class=>"fedora-radio-button", :rel=>h_name, :name=>"asset[#{datastream_name}][#{h_name}][0]", :value=>choice.downcase, :checked=>true)
        else
          result << tag(:input, :type=>"radio", :id=>"availability_#{choice}", :class=>"fedora-radio-button", :rel=>h_name, :name=>"asset[#{datastream_name}][#{h_name}][0]", :value=>choice.downcase)
        end
        result << "<label for='availability_#{choice}'>#{label}</label>"
      end
      result
    end
    return result
  end
  
  
  def fedora_select(resource, datastream_name, field_key, opts={})
    if opts[:choices].nil?
      result = fedora_text_field(resource, datastream_name, field_key, opts)
    else
      choices = opts[:choices]
      my_choices = []
      if opts[:default_choice]
        my_choices << opts[:default_choice]
      end
      
      if opts.fetch(:label, false)
        label = opts.fetch(:label)
      else
        label = ""
      end

      if choices.is_a?(Hash)
        choice_order = choices.keys.sort!
        choice_order.each do |choice|
          my_choices << [choice, choices[choice]]
        end
      else 
        my_choices << choices
        my_choices = my_choices.flatten
      end
      
      field_name = field_name_for(field_key)
      field_values = get_values_from_datastream(resource, datastream_name, field_key, opts)
      
      body = ""
      z = 0
      base_id = generate_base_id(field_name, field_values.first, field_values, opts.merge({:multiple=>false}))
      name = "asset[#{datastream_name}][#{field_name}][#{z}]"
      body << "<label for \"#{name}\">#{label}</label>"
      body << "<select name=\"#{name}\" class=\"metadata-dd select-edit\" rel=\"#{field_name}\">"
        body << options_for_select(my_choices, field_values)
      body << "</select>"
      
      result = field_selectors_for(datastream_name, field_key)
      result << body
    end
    return result
  end
  
  
  def has_values?(resource, datastream_name, field_key, opts={})
    has_node = true if !get_values_from_datastream(resource, datastream_name, field_key, opts).empty? rescue false
    if has_node
      val = get_values_from_datastream(resource, datastream_name, field_key, opts).first rescue ""
      return !val.blank?
    end
    return false
  end
  
end
