require 'prawn'

module LibraHelper
  
  def display_release_status_notice(document)
    readiness = document.test_release_readiness
    Rails.logger.info("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@")
    Rails.logger.info("@@@@@@@@@@@ Release Readiness is = #{readiness}  @@@@@@@@@@@@@@@@@")
    if readiness != true
      flash[:error] ||= []
      failures = []
      readiness[:failures].each do |failure|
        failures << "<p>#{failure}</p>"
      end
      flash[:error] = flash[:error] | failures
    end
  end
  
  def edit_and_browse_links
    result = ""
    view_text = @document.get(:released_facet) === "true" ? "View" : "Preview"
    if params[:action] == "edit"
      result << "<a href=\"#{catalog_path(@document[:id], :viewing_context=>"browse")}\" class=\"browse toggle\">#{view_text}</a>"
      result << "<span class=\"edit toggle active\">Edit</span>"
    else
      result << "<span class=\"browse toggle active\">#{view_text}</span>"
      result << "<a href=\"#{edit_catalog_path(@document[:id])}\" class=\"edit toggle\">Edit</a>"
    end
    # result << link_to "Browse", "#", :class=>"browse"
    # result << link_to "Edit", edit_document_path(@document[:id]), :class=>"edit"
    return result
  end

  def license_choices
    choices = {}
    etd = @document_fedora.class.respond_to?(:sis_mapping)
    if etd
      if (ASG_TEMPLATE.include?(academic_program(@document_fedora)))
        choices["uetd"] = "I agree to the Libra Deposit License for Student Theses and Dissertations.  My work will be available to the UVA community immediately.  My work will be available for worldwide open access according to the release date I choose below.<span id='uetd_read' class='license_toggle'>Read License</span>"
      else
        choices["uetd"] = "Make my work available to everyone. I agree to the Libra Deposit License for Student Theses and Dissertations. <span id='uetd_read' class='license_toggle'>Read License</span>"
      end
    end
    if (etd and current_user.can_be_superuser?) or (!etd and !@document_fedora.dataset?)
      choices["uva"] = "Make my work available only to U.Va.-authenticated users. I agree to the Libra U.Va.-Only Deposit License. <span id='uva_read' class='license_toggle'>Read License</span>"
    end
    if !etd and !@document_fedora.dataset?
      choices["public"] = "Make my work available to everyone. I agree to the Libra Public Deposit License. <span id='public_read' class='license_toggle'>Read License</span>"
    end
    if @document_fedora.dataset?
      choices["public_dataset"] = "Make my work available to everyone. I agree to the Libra Public Dataset License. <span id='dataset_read' class='license_toggle'>Read License</span>"
    end
    choices["none"] = "I do not agree to the U.Va. Libra Submission Agreement."
    choices
  end
  
  def role_string(fedora_doc, person_counter)
    role = ""
    role_node = get_values_from_datastream(fedora_doc, "descMetadata", [{:person=>person_counter}, :role])
    if role_node and role_node.first and !role_node.first.empty?
      role = get_values_from_datastream(fedora_doc, "descMetadata", [{:person=>person_counter}, :role, :text])   
    end
    role
  end    
  
  def name_string(fedora_doc, person_counter)
    last_name = ""
    first_name = ""
    description = ""
    institution = ""
    last_name_node = get_values_from_datastream(fedora_doc, "descMetadata", [{:person=>person_counter}, :last_name])
    if last_name_node and last_name_node.first and !last_name_node.first.empty?
      last_name = last_name_node.first.strip
    end
    first_name_node = get_values_from_datastream(fedora_doc, "descMetadata", [{:person=>person_counter}, :first_name])
    if first_name_node and first_name_node.first and !first_name_node.first.empty?
      first_name = first_name_node.first.strip
    end
    description_node = get_values_from_datastream(fedora_doc, "descMetadata", [{:person=>person_counter}, :description])
		if description_node and description_node.first and !description_node.first.empty?
		  description = description_node.first.strip
		  description = department_facet_mapping(description)
    end
    institution_node = get_values_from_datastream(fedora_doc, "descMetadata", [{:person=>person_counter}, :institution])
    if institution_node and institution_node.first and !institution_node.first.empty?
      institution = institution_node.first.strip
    end 
    out = ""
    out += last_name
    out += ", " unless (last_name.blank? or first_name.blank?)
    out += first_name unless first_name.blank?
    out += ", " + description unless description.blank?
    out += ", " + institution unless institution.blank?
    out
  end
  
  def etd_object_type_facet_values
    [ HydrangeaFourthYearThesis.solr_object_type_facet, 
      HydrangeaDoctoralDissertation.solr_object_type_facet, 
      HydrangeaMastersThesis.solr_object_type_facet ]
  end
  
  def etd_solr_type?(solr_doc)
    etd_object_type_facet_values.include?(solr_doc.get(:object_type_facet))
  end
  
  def description(solr_doc)
    return unless etd_solr_type?(solr_doc)
    description = solr_doc.get(:mods_0_person_0_description_t) || ""
    department_facet_mapping(description)
  end
  
  # looks to shee if the provided department has an alternate display version
  def department_facet_mapping(department)
    mapping = DepartmentFacetMapping.find_by_department_facet(department)
    mapping.nil? ? department : mapping.department_display_facet
  end
  
  def embargo_radio_buttons(fedora_doc)

   return if !fedora_doc.class.respond_to?(:sis_mapping)
   
   #Add title to find the right embargo months when a person submit two master thesis
   etd_title = get_values_from_datastream(fedora_doc, "descMetadata", [:mods, :title_info, :main_title])
   record = SisRecord.find_by_computing_id_and_milestone_and_title_of_work(fedora_doc.depositor, fedora_doc.class.sis_type, etd_title)

   return if record.nil?
   
   #Display embargo radio button for user when doc is not released or for superuser when it's released
   name = "asset[sisMetadata][:embargo_selection][0]"
   academic_program = record.academic_program

   #Do not display preselected radio button for ASG etds
   if record.libra_id.nil? && record.embargo_months == 0 && ASG_TEMPLATE.include?(academic_program)
      months = -1
   else
      months = record.embargo_months      
   end

   embargo_node = fedora_doc.datastreams['rightsMetadata'].find_by_terms(:embargo, :embargo_release_date) unless fedora_doc.datastreams['rightsMetadata'].nil?

   # At this point, the embargo radio buttons will be shown.  The question is whether they will be enabled or disabled.
   # They will be disabled if
   #        1) the Fedora doc is already marked as released
   #    and 2) the embargo has already been applied
   #    and 3) the user is not currently a superuser.
   options = {:class => 'fedora-radio-button'}
   if fedora_doc.is_released? and embargo_node and embargo_node[0] and !current_user.is_being_superuser?(session)
     options = {:class => 'fedora-radio-button', :disabled => 'disabled'}
   end
   out = ""
      out += label_tag("embargo_selection_0", "#{radio_button_tag(name, 0, months == 0, options)} Make available for worldwide access immediately", :class => 'embargo-radio-label')
      out += label_tag("embargo_selection_6", "#{radio_button_tag(name, 6, months == 6, options)} Make available for worldwide access 6 months from now", :class => 'embargo-radio-label')
      out += label_tag("embargo_selection_12", "#{radio_button_tag(name, 12, months == 12, options)} Make available for worldwide access 1 year from now", :class => 'embargo-radio-label')
      out += label_tag("embargo_selection_24", "#{radio_button_tag(name, 24, months == 24, options)} Make available for worldwide access 2 years from now", :class => 'embargo-radio-label')
      out += label_tag("embargo_selection_60", "#{radio_button_tag(name, 60, months == 60, options)} Make available for worldwide access 5 years from now", :class => 'embargo-radio-label')

   # If the Fedora doc is marked as released and it has an embargo, show that embargo release date above the embargo radio buttons.
   if fedora_doc.is_released? and embargo_node and embargo_node[0]
     return "<br /><br />Embargo Release Date:  " + embargo_node[0].content + "<br /><br />" + out
   end

   out

  end
  
  def publication_year_choices(future_years = 0)
    first = 1900
    last = Time.now.year + future_years
    (first..last).to_a.reverse.map { |i| i.to_s }
  end
  
  def libra_submission_status(document)
    sis_record = SisRecord.find_by_libra_id(document[:id])
    return "" if sis_record.blank?
    if sis_record.libra_wrote_to_sis.blank?
      return "You have successfully submitted your #{document[:object_type_facet]} to Libra on #{sis_record.libra_approved.strftime('%B %d, %Y')}. Please note that it may take up to 24 hours for your status to be updated in SIS."
    else
      return "This #{document[:object_type_facet]} was successfully submitted to SIS on #{sis_record.libra_wrote_to_sis.strftime('%B %d, %Y')}." 
    end
  end

  # Checks to see if the Solr document is a Hydrangea Dataset or not
  def is_dataset?(solr_doc)
    return solr_doc.get(:object_type_facet).downcase == "dataset" rescue false
  end
  
  def is_etd?(solr_doc)
    return solr_doc.get(:object_type_facet).downcase == "doctoral dissertation" || solr_doc.get(:object_type_facet).downcase == "master's thesis" || solr_doc.get(:object_type_facet).downcase == "fourth year thesis" rescue false
  end

  def search_results_year_row(document)
    return "" if document[:origin_info_t].blank?
    document.get(:origin_info_t).to_time.year rescue ""
  end
  
  # help text for the author field.  Add new verbiage according to types in app/models/
  def author_help_text(document)
    return "A person responsible for the collection and/or assembly of this data" if document.class == HydrangeaDataset
    "A creator of this work." # default
  end

  # help text for the title field.  Add new verbiage according to types in app/models/  
  def title_help_text(document)
    return "A brief descriptive title for this collection of data" if document.class == HydrangeaDataset
    "The title of this work" # default
  end
  
  # help text for the keywords field.  Add new verbiage according to types in app/models/
  def keywords_help_text(document)
    "One or more comma-separated labels which may help make your work discoverable by others by indicating the topic(s) with which it is associated" # default
  end
  
  #help text for upload files
  def upload_help_text(document)
    "One and only one thesis or dissertation file in pdf format must be uploaded.  In addition, you have the option of adding supplemental files such as images or datasets to accompany the thesis or dissertation.  Supplemental files can be in any format, however, the use of common file formats is recommended."
  end
  
  # help text for embargo.  Add new verbiage according to types in app/models/
  def embargo_help_text(document)
    etd = @document_fedora.class.respond_to?(:sis_mapping)
    return "By default your work will be made available to Libra users immediately. 
    If you need to delay the release of your work, please choose a date in the future from the calendar. 
    Refer to the FAQs for further information about why you may want to delay release of your work." if document.class == HydrangeaDataset
    
    return "Your work will be available to the UVA community.  
    You can choose to make it available for worldwide open access at this time, or you may choose an embargo period after which it will be released worldwide.  
    Please see the FAQs for more information about when and why you might want to use an embargo." if (etd and academic_program(document) == "ASG")
    
    "By default your work will be made available to Libra users immediately.  
    If you need to delay the release of your work, please choose a date from the available options.  
    Refer to the FAQs for further information about why you may want to delay release of your work." # default
  end

  # A hack-y way of sorting files by arbitrary file type values. Need a better way of doing this.
  def file_type_sort(file)
    file_type = file[solr_name("description", :text)].to_s
    
    if file_type == "readme" || file_type == "thesis"
      sort_value = 1
    elsif file_type == "dataset" || file_type == "supplement"
      sort_value = 2
    else
      sort_value = 3
    end

    sort_value
  end
  
  # A method returns true if this ETD comes from the academic program that uses ASG template
  def is_ASG?(document)
    ASG_TEMPLATE.each { |academic_program|
      return true if SisRecord.find(:all, :conditions => ["libra_id = ? AND academic_program IN (?)", document[:id], academic_program]).count > 0
    }
    return false
  end
  
  def academic_program(fedora_doc)
    return if !fedora_doc.class.respond_to?(:sis_mapping)
    etd_title = get_values_from_datastream(fedora_doc, "descMetadata", [:mods, :title_info, :main_title])
    record = SisRecord.find_by_computing_id_and_milestone_and_title_of_work(fedora_doc.depositor, fedora_doc.class.sis_type, etd_title)

    return if record.nil?
    return record.academic_program
  end

  def has_one_thesis?
    value = false
    if !@document[:id].nil?
      @container =  ActiveFedora::Base.load_instance(@document[:id])
      @response, @document = get_solr_response_for_doc_id(@document[:id])
      @solr_result = @container.file_objects({:response_format=>:solr, :rows=>UVA_SOLR_ROWS_NUM})
      unless @solr_result.hits.empty?
        @solr_result.hits.each do |file|
          if file["description_t"].to_s == "thesis"
            value = true
          end
        end
      end
    end
    return value
  end


  ######################################################################################
  #
  # Use gem Prawn to create pdf cover dynamically when users click to download pdf files
  #
  ######################################################################################

  def create_pdf_cover(pdf_file)
    cover_file = "#{RAILS_ROOT}/tmp/pdf_covers/cover_of_#{pdf_file.label}"
    disclaimer, restricted_access = display_disclaimer(@document)
    Prawn::Document.generate(cover_file,
                              :info => {
                                  :Title => pdf_item_title,
                                  :Author => pdf_author_fullname,
                                  :Keywords => pdf_info_keywords
                              }) do |cover|

      cover.image "#{pdf_uva_logo}", :width =>500, :height => 55
      cover.move_down 15
      cover.font_size 14
      cover.text pdf_item_title, :style => :bold
      cover.move_down 15
      cover.font_size 10
      if restricted_access == true
        cover.text disclaimer, :style => :bold
      else
        cover.text disclaimer
      end
      cover.move_down 15
      data = [["<font size='12'>APA Citation:</font> ", "<font size='10'>#{display_citation}</font>"],
              ["<font size='12'>Accessed:</font> ", "<font size='10'>#{pdf_accessed_date}</font>"],
              ["<font size='12'>Permanent URL:</font>  ", "<font size='10'>#{pdf_item_permanent_url}</font>"],
              ["<font size='12'>Keywords:</font> ", "<font size='10'>#{pdf_linked_keyword}</font>"],
              ["<font size='12'>Terms:</font> ", "<font size='10'>#{pdf_terms}</font>"]
      ]
      cover.table( data, :cell_style => { :border_width => 0, :inline_format => true })
      cover.move_down 20
      cover.text "(Article begins on next page)", :align => :center
    end
    return cover_file
  end

  # UVA logo on pdf cover
  def pdf_uva_logo
    return File.join(Rails.root, '/public/images/uva/uva_logo.jpg')
  end

  # Year user download the pdf file, format: "month dd, yyyy"
  def pdf_accessed_date
    return Date.parse(Time.now.to_s).strftime("%B %d, %Y")
  end

  # Title displayed on pdf cover
  def pdf_item_title
    return @document.get(:mods_0_title_info_t) unless @document.get(:mods_0_title_info_t).nil?
  end

  # Permanent URL for the ETD or other Libra items
  def pdf_item_permanent_url
    return "<u><link href = '" + LIBRA_HOST + "/catalog/#{@document.id}'>" + LIBRA_HOST + "/catalog/#{@document.id}</link></u>"
  end

  # Author name displayed in pdf metadata
  def pdf_author_fullname
    name_list = []
    num = @document.get(:name_first_name_t).split(",").length unless @document.get(:name_first_name_t).nil?
    for i in 0..(num-1)
      unless @document.get("mods_0_person_#{i}_role_t".to_sym).nil?
        if @document.get("mods_0_person_#{i}_role_t".to_sym).include?("author")
          name_list << " & " unless name_list.empty?
          name_list << @document.get("mods_0_name_#{i}_last_name_t".to_sym) unless @document.get("mods_0_name_#{i}_last_name_t".to_sym).nil?
          name_list << ", "
          name_list << @document.get("mods_0_name_#{i}_first_name_t".to_sym) unless @document.get("mods_0_name_#{i}_first_name_t".to_sym).nil?
        end
      end
    end
    return name_list
  end

  # Author name displayed in APA citataion
  def pdf_author_shortname
    name_list = []
    num = @document.get(:name_first_name_t).split(",").length unless @document.get(:name_first_name_t).nil?
    for i in 0..(num-1)
      unless @document.get("mods_0_person_#{i}_role_t".to_sym).nil?
      if @document.get("mods_0_person_#{i}_role_t".to_sym).include?("author")
        name_list << " & " unless name_list.empty?
        name_list << @document.get("mods_0_name_#{i}_last_name_t".to_sym) unless @document.get("mods_0_name_#{i}_last_name_t".to_sym).nil?
        name_list << ", "
        name_list << @document.get("mods_0_name_#{i}_first_name_t".to_sym)[0,1] unless @document.get("mods_0_name_#{i}_first_name_t".to_sym).nil?
      end
        end
    end
    return name_list
  end

  # Keywords displayed in pdf metadata
  def pdf_info_keywords
    return @document.get(:topic_tag_t).nil? ? "" : @document.get(:topic_tag_t).split(",")
  end

  # Keywords displayed on pdf cover, should be a link of Libra keyword search
  def pdf_linked_keyword
    linked_keyword = ""
    pdf_info_keywords.each do |kw|
      #linked_keyword << "<u><link href = '" + LIBRA_HOST + "/catalog?q=#{kw.strip}'>" + LIBRA_HOST + "/catalog?q=#{kw.strip}</link></u>\n"
      linked_keyword << "<u><link href = '" + LIBRA_HOST + "/catalog?q=#{kw.strip}'>#{kw.strip}</link></u> "
    end
    return linked_keyword
  end

  # Terms displayed on pdf cover
  def pdf_terms
    return "This article was downloaded from the University of Virginia’s Libra institutional repository, and is made available under the terms and conditions applicable as set forth at <u><link href = '" + LIBRA_HOST + "/terms'>" + LIBRA_HOST + "/terms</link></u>"
  end

  # Select right disclaimer displayed on pdf cover. Text for the "disclaimer" depending on the status of the work; in the case of restricted access, the font should be bold
  def display_disclaimer(doc)
    restricted_access = true
    if !doc["embargo_release_date_dt"].blank? && during_embargo?
      embargo_date = Date.parse(doc["embargo_release_date_dt"].split(/T/)[0])
      if is_ASG?(doc)
        disclaimer_text = "Distribution of this item is restricted to members of the University of Virginia community until #{embargo_date}."
      else
        disclaimer_text = "THIS ITEM IS UNDER EMBARGO UNTIL #{embargo_date} AND MAY ONLY BE ACCESSED BY ITS AUTHOR PRIOR TO THAT DATE. IF YOU HAVE RECEIVED THIS FILE WITHOUT EXPRESS PERMISSION OF ITS AUTHOR, PLEASE DESTROY IT IMMEDIATELY."
      end
    elsif !doc['release_to_t'].nil? && doc['release_to_t'].first == 'uva'
      disclaimer_text = "Distribution of this item is restricted to members of the University of Virginia community."
    else
      disclaimer_text = "The author of this item has granted worldwide open access to this work."
      restricted_access = false
    end
    return disclaimer_text, restricted_access
  end

  # Generate APA citation
  def display_citation
    creation_date = @document.get(:year_i)
    journal = @document.get(:journal_title_info_main_title_t)
    book_title = @document.get(:book_0_title_info_0_main_title_t).nil? ? "" : @document.get(:book_0_title_info_0_main_title_t)
    volume = @document.get(:journal_issue_volume_t).nil? ? "" : @document.get(:journal_issue_volume_t)
    issue = @document.get(:journal_issue_level_t).nil? ? "" : @document.get(:journal_issue_level_t)
    start_page = @document.get(:journal_issue_pages_start_t)
    end_page = @document.get(:journal_issue_pages_end_t)
    editors = ""
    if !@document.get(:book_editor_t).nil?
      list = @document.get(:book_editor_t).split(",")
      list.each{ |x|
        editors << ", " unless editors.empty?
        editors  << x.split(" ").take(2).join(" ")
      }
    end

    citation = "#{pdf_author_shortname}.(#{creation_date}). #{pdf_item_title}. "
    citation += "In " + editors + " (Eds.), " unless editors.blank?
    citation += "<i>#{book_title}. </i> " unless book_title.blank?
    citation += "<i>#{journal}. </i> " unless journal.blank?
    citation += "<i>#{volume}</i>" unless volume.blank?
    citation += "(#{issue}), " unless issue.blank?
    citation += "#{start_page}-#{end_page}. " if !start_page.blank? || !end_page.blank?
    citation += "Retrieved from #{pdf_item_permanent_url}"
    return citation
  end

  # Create the new name for pdf with auto generated cover
  # Name format: Gibson_2013_UVa_libra-oa_56 (Author, date, "UVa", PID with special characters replaced by underscore
  # including Ensure that (* < > [ ] = + ” \ / , . : ; ? % # $ | & •)
  def pdf_new_name
    time = Time.now.to_i.to_s
    author_lastname = @document.get(:name_0_last_name_t)
    year = Date.parse(Time.now.to_s).strftime("%Y")
    # We should use file asset's id instead of document id here
    pdf_pid = @file_asset.pid.gsub(/[^0-9A-Za-z\-]/,'_')
    return author_lastname + "_" + year + "_UVa_" + pdf_pid + "_" + time + ".pdf"
  end

  #
  # This use the PDFtk to combine PDF files
  # PDFtk website: https://www.pdflabs.com/tools/pdftk-the-pdf-toolkit/
  #
  def pdf_combine(cover, pdf, pdf_new_name)
    output_file = File.join(Rails.root, "/tmp/pdf_covers/#{pdf_new_name}")
    empty_pdf_page = "#{RAILS_ROOT}/tmp/empty_pdf_page.pdf"
    `pdftk "#{cover}" "#{pdf}" output "#{output_file}"`
    `pdftk "#{cover}" "#{empty_pdf_page}" output "#{output_file}"` unless $?.success?
    return output_file
  end

  # This method use prawn to combine pdf with it's cover. It's not working under ruby 1.8.7 environment
  # and need to be tested under ruby 1.9.3
  # def prawn_combine(cover, orig_pdf, pdf_new_name)
  #   pdf_file_paths = [cover, orig_pdf]
  #   Prawn::Document.generate("#{RAILS_ROOT}/tmp/pdf_covers/#{pdf_new_name}", {:page_size => 'A4', :skip_page_creation => true}) do |pdf|
  #     pdf_file_paths.each do |pdf_file|
  #       if File.exists?(pdf_file)
  #         pdf_temp_nb_pages = Prawn::Document.new(:template => pdf_file).page_count
  #         (1..pdf_temp_nb_pages).each do |i|
  #           pdf.start_new_page(:template => pdf_file, :template_page => i)
  #         end
  #       end
  #     end
  #   end
  #   return "#{RAILS_ROOT}/tmp/pdf_covers/#{pdf_new_name}"
  # end
end
