class HydrangeaArticle < ActiveFedora::Base

  include Hydra::ModelMethods
  include Uva::CommonModelMethods
  
  has_relationship "parts", :is_part_of, :inbound => true
  
  # Uses the Hydra Rights Metadata Schema for tracking access permissions & copyright
  has_metadata :name => "rightsMetadata", :type => Hydra::RightsMetadata 
  
  # Uses the Hydra MODS Article profile for tracking most of the descriptive metadata
  has_metadata :name => "descMetadata", :type => Hydra::ModsArticle 

  # A place to put extra metadata values
  has_metadata :name => "properties", :type => ActiveFedora::MetadataDatastream do |m|
    m.field 'collection', :string
    m.field 'depositor', :string
    m.field 'released', :string
    m.field 'release_to', :string
    m.field 'peer_reviewed', :string
  end
  
  def self.solr_object_type_facet
    "Article"
  end
  
  # Tests whether the object is ready to release
  # @return true if ready, otherwise a Hash where response[:failures] is an array of failure messages
  def test_release_readiness
    author_entries = datastreams["descMetadata"].find_by_terms_and_value('//oxns:name[@type="personal" and contains(oxns:role, "author") and string-length(oxns:namePart[@type="family"]) > 0 ]')    
    title_values = datastreams["descMetadata"].term_values(:title_info, :main_title)
    journal_title_values = datastreams["descMetadata"].term_values(:journal, :title_info, :main_title)
    published_year_values = datastreams["descMetadata"].term_values(:journal, :issue, :publication_year)
    release_to_values = datastreams["properties"].release_to_values
    response = {:failures=>[]}

    if title_values.empty? || title_values.first.empty?
      response[:failures] << "Please provide a title."
    end  
    if author_entries.empty?
      response[:failures] << "Please provide an author (first name and last name)."
    end  
    if journal_title_values.empty? || journal_title_values.first.empty?
      response[:failures] << "Please provide a journal title."
    end
    if parts(:response_format=>:id_array).empty?
      response[:failures] << "Please attach at least one file."
    end
    if published_year_values.empty? || published_year_values.first.empty?
      response[:failures] << "Please provide a publication year"
    end
    if release_to_values.empty? || !["uva", "public"].include?(release_to_values.first)
      response[:failures] << "You must agree to the U.Va. Libra Submission Agreement."
    end
    
    if response[:failures].empty? 
      return true
    else
      return response
    end
  end

end
