require 'vendor/plugins/blacklight/app/helpers/application_helper.rb'
require 'vendor/plugins/hydra_repository/app/helpers/application_helper.rb'
require 'lib/uva/config.rb'

module ApplicationHelper

  # Include the Uva constants in the compiled *.html.erb files -- Rails 2 way.
  def self.included(base)
    base.send :include, Uva::Config
  end

  def render_facet_value(facet_solr_field, item, options ={})
    if item.is_a? Array
      link_to_unless(options[:suppress_link], item[0], add_facet_params_and_redirect(facet_solr_field, item[0]), :class=>"facet_select") + " (" + format_num(item[1]) + ")" 
    else
      link_to_unless(options[:suppress_link], item.value, add_facet_params_and_redirect(facet_solr_field, item.value), :class=>"facet_select") + " (" + format_num(item.hits) + ")" 
    end
  end
  
  # Removing the [remove] link from the default selected facet display
  def render_selected_facet_value(facet_solr_field, item)
    '<span class="selected">' +
    render_facet_value(facet_solr_field, item, :suppress_link => true) +
    '</span>'
  end

  def render_complex_facet_value(facet_solr_field, item, options ={})    
    link_to_unless(options[:suppress_link], format_item_value(item.value), add_facet_params_and_redirect(facet_solr_field, item.value), :class=>"facet_select") + " (" + format_num(item.hits) + ")" 
  end
  
  def render_journal_facet_value(facet_solr_field, item, options ={})
    
    val = item.value.strip.length > 12 ? item.value.strip[0..12].concat("...") : item.value.strip
    link_to_unless(options[:suppress_link], val, add_facet_params_and_redirect(facet_solr_field, item.value), :class=>"facet_select") + " (" + format_num(item.hits) + ")" 
  end

  def render_complex_facet_image(facet_solr_field, item, options = {})
    computing_id = extract_computing_id(item.value)
    if File.exists?("#{Rails.root}/public/images/faculty_images/#{computing_id}.jpg")
      img = image_tag "/images/faculty_images/#{computing_id}.jpg", :width=> "100", :alt=>"#{item.value}"
    elsif   Ldap::Person.new(computing_id).has_photo?
      img = image_tag ldap_photo_path(computing_id), :width=>"100", :alt=>"#{item.value}"
    else
      img = image_tag "/plugin_assets/hydra_repository/images/default_thumbnail.gif", :width=>"100", :alt=>"#{item.value}"
    end
    link_to_unless(options[:suppress_link], img, add_facet_params_and_redirect(facet_solr_field, item.value), :class=>"facet_select facet_image") 
  end

  def render_journal_image(facet_solr_field, item, options = {})
    if File.exists?("#{Rails.root}/public/images/journal_images/#{item.value.strip.downcase.gsub(/\s+/,'_')}.jpg")
      img = image_tag "/images/journal_images/#{item.value.strip.downcase.gsub(/\s+/,'_')}.jpg", :width => "100"
    else
      img = image_tag "/plugin_assets/hydra_repository/images/default_thumbnail.gif", :width=>"100", :alt=>"#{item.value}"
    end

    link_to_unless(options[:suppress_link], img, add_facet_params_and_redirect(facet_solr_field, item.value), :class=>"facet_select") 
  end

  def get_randomized_display_items items
    clean_items = items.each.inject([]) do |array, item|
      array << item unless item.value.strip.blank?
      array
    end

    if clean_items.length < 6 
      clean_items.sort_by {|item| item.value }
    else 
      rdi = clean_items.sort_by {rand}.slice(0..5)
      return rdi.sort_by {|item| item.value.downcase}
    end
    
  end

  def extract_computing_id val
    cid = val.split(" ")[-1]
    cid[1..cid.length-2]
  end

  def format_item_value val
    begin
      last, f_c = val.split(", ")
      first = f_c.split(" (")[0]
    rescue
      return val.nil? ? "" : val
    end
    [last, "#{first[0..0]}."].join(", ")
  end

#   COPIED from vendor/plugins/blacklight/app/helpers/application_helper.rb
  # Used in catalog/facet action, facets.rb view, for a click
  # on a facet value. Add on the facet params to existing
  # search constraints. Remove any paginator-specific request
  # params, or other request params that should be removed
  # for a 'fresh' display. 
  # Change the action to 'index' to send them back to
  # catalog/index with their new facet choice. 
  def add_facet_params_and_redirect(field, value)
    new_params = add_facet_params(field, value)

    # Delete page, if needed. 
    new_params.delete(:page)

    # Delete :qt, if needed - added to resolve NPE errors
    new_params.delete(:qt)

    # Delete any request params from facet-specific action, needed
    # to redir to index action properly. 
    Blacklight::Solr::FacetPaginator.request_keys.values.each do |paginator_key| 
      new_params.delete(paginator_key)
    end
    new_params.delete(:id)

    # Force action to be index. 
    new_params[:action] = "index"

    new_params
  end

  def application_name
     'Libra'
  end

  def default_page_title
    "#{application_name} - University of Virginia Library"
  end

  def show_page_title(title)
    if RAILS_ENV == "production"
      app_v = ""
    else
      app_v = APP_VERSION
    end

    if title.nil? || title.blank?
      return "#{default_page_title} #{app_v}"
    else
      return "#{title} - University of Virginia Library - #{application_name} #{app_v}"
    end
  end

  
  def item_deleteable?(released_values)
    return true if current_user.is_being_superuser?(session)
    released_values[0] == "true" ? false : true rescue false
  end

  def item_released?(released_values)
    return released_values[0] == "true" rescue false
  end

  def sort_fields
    sort_fields = []
    Blacklight.config[:sort_fields_order].each { |key|
      sort_fields << [Blacklight.config[:sort_fields][key][0], key]
    }
    # remove 'relevancy' if there is no search
    sort_fields.shift if params[:q].blank?
    sort_fields
  end
  
  def build_facet_params(facet_pairs)
    p = {}
    facet_pairs.each do |key, value|
      p[:f]||={}
      p[:f][key] ||= []
      p[:f][key].push(value)
    end
    p
  end
  
  # used in index table
  def author_display(document, default="n/a")
    author = get_person_from_role(document, "aut author")||get_person_from_role(document, "autauthor")||get_person_from_role(document, "dtc Data contributor")||get_person_from_role(document, "dtcData contributor")
    return default if author.nil?
    "#{author[:last]}, #{author[:first]}"
  end

  # Genrates a Z39.88 span element to pass the COinS metadata to Zotero
  def zotero_span(document,format)
    title = get_values_from_datastream(document,"descMetadata", [:title_info, :main_title]).first rescue ""
    
    author = get_values_from_datastream(document,"descMetadata",[{:person=>0}, :first_name]).first rescue ""
    author += " "
    author += get_values_from_datastream(document,"descMetadata",[{:person=>0}, :last_name]).first rescue ""
    
    if format == 'journal'  
      journal_title = get_values_from_datastream(document,"descMetadata", [:journal, :title_info, :main_title]).first rescue ""
      issue = get_values_from_datastream(document,"descMetadata", [:journal, :issue, :level, :number]).first rescue ""
      volume = get_values_from_datastream(document,"descMetadata", [:journal, :issue, :volume, :number]).first rescue ""
      start_page = get_values_from_datastream(document,"descMetadata", [:journal, :issue, :start_page]).first rescue ""
      end_page = get_values_from_datastream(document,"descMetadata", [:journal, :issue, :end_page] ).first rescue ""

      issn = get_values_from_datastream(document, "descMetadata", [:journal, :issn]).first rescue ""
    else
      publication_date = get_values_from_datastream(document, "descMetadata", [:origin_info, :date_issued]).first rescue ""
      isbn = get_values_from_datastream(document, "descMetadata", [:isbn]).first rescue ""
    end

    format.is_a?(Array) ? format = format[0].downcase.strip : format = format.downcase.strip
    text = "<span class='Z3988' "
    if format == 'book'
      text << "title='ctx_ver=Z39.88-2004&amp;rft_val_fmt=info%3Aofi%2Ffmt%3Akev%3Amtx%3Abook&amp;rfr_id=info%3Asid%2Fblacklight.rubyforge.org%3Agenerator&amp;rft.genre=book&amp;rft.btitle=#{(title.nil?) ? "" : CGI::escape(title)}&amp;rft.au=#{(author.nil?) ? "" : CGI::escape(author)}&amp;rft.date=#{(publication_date.nil?) ? "" : CGI::escape(publication_date)}&amp;rft.isbn=#{(isbn.nil?) ? "" : isbn}'"
    elsif format.include?('journal') # checking using include because institutions may use formats like Journal or Journal/Magazine
      text << "title='ctx_ver=Z39.88-2004&amp;rft_val_fmt=info%3Aofi%2Ffmt%3Akev%3Amtx%3Ajournal&amp;rfr_id=info%3Asid%2Fblacklight.rubyforge.org%3Agenerator"
      text << "&amp;rft.genre=article"
      text << "&amp;rft.title=#{(journal_title.nil?) ? "" : CGI::escape(journal_title)}"
      text << "&amp;rft.atitle=#{(title.nil?) ? "" : CGI::escape(title)}"
      text << "&amp;rft.au=#{(author.nil?) ? "" : CGI::escape(author)}"
      text << "&amp;rft.date=#{(publication_date.nil?) ? "" : CGI::escape(publication_date)}"
      text << "&amp;rft.issn=#{(issn.nil?) ? "" : CGI::escape(issn)}"
      text << "&amp;rft.issue=#{(issue.nil?) ? "" : CGI::escape(issue)}"
      text << "&amp;rft.volume=#{(volume.nil?) ? "" : CGI::escape(volume)}"
      text << "&amp;rft.spage=#{(start_page.nil?) ? "" : CGI::escape(start_page)}"
      text << "&amp;rft.epage=#{(end_page.nil?) ? "" : CGI::escape(end_page)}"
      text << "'"
    end
    text << "></span>"
  end

  
end
