module Hydra
  require_dependency 'vendor/plugins/hydra_repository/lib/hydra/mods_article.rb'
  
  class ModsArticlePreprint < ActiveFedora::NokogiriDatastream       
    
    include Uva::CommonTermMethods
    
    set_terminology do |t|
      set_basics(t)
      set_sponsor(t)

      t.journal(:path=>'relatedItem', :attributes=>{:type=>"host"}) {
        t.title_info(:index_as=>[:facetable],:ref=>[:title_info])
        t.origin_info(:path=>"originInfo") {
          t.publisher
          t.date_issued(:path=>"dateIssued")
          t.issuance(:index_as=>[:facetable])
        }
        t.issn(:path=>"identifier", :attributes=>{:type=>"issn"})
        t.issue(:path=>"part") {
          t.volume(:path=>"detail", :attributes=>{:type=>"volume"}) {
            t.number
          }
          t.level(:path=>"detail", :attributes=>{:type=>"number"}) {
            t.number
          }
          t.extent
          t.pages(:path=>"extent", :attributes=>{:unit=>"pages"}) {
            t.start
            t.end
          }
          t.start_page(:proxy=>[:pages, :start])
          t.end_page(:proxy=>[:pages, :end])
          t.publication_date(:path=>"date", :attributes=>{:keydate=>"yes"})
          t.publication_year(:path=>"date", :attributes=>{:keydate=>:none}, :index_as=>[:facetable])
        }
      }
      t.note
      t.other_version(:path=>"relatedItem", :attributes=>{:type=>"otherVersion"}) {
        t.location(:path=>"location") {
          t.url(:path=>"url")
        }
      }
      t.publication_url(:proxy=>[:other_version,:location,:url])
      t.peer_reviewed(:proxy=>[:journal,:origin_info,:issuance], :index_as=>[:facetable])
    end
    
    # Generates an empty Mods Article Preprint (used when you call ModsArticle.new without passing in existing xml)
    def self.xml_template
      build_template(:article_preprint)
    end
    
    # Generates a new Person node
    def self.person_template
      build_person(:article_preprint)
    end
    
    def to_solr(solr_doc=Solr::Document.new)
      solr_doc = super(solr_doc)
      build_solr(solr_doc)
      solr_doc << {:object_type_facet => HydrangeaArticlePreprint.solr_object_type_facet }        
      solr_doc << Solr::Field.new(:year_i => solr_doc["journal_issue_publication_year_facet"]) if solr_doc["journal_issue_publication_year_facet"]
      solr_doc
    end
    
  end
end
