module Hydra
    
  class ModsBook < ActiveFedora::NokogiriDatastream       

    include Uva::CommonTermMethods

    set_terminology do |t|
      set_basics(t)
      set_sponsor(t)
      
      t.isbn(:path=>"identifier", :attributes=>{:type=>"isbn"})
      t.origin_info(:path=>"originInfo") {
        t.publisher
        t.place {
          t.place_term(:path=>"placeTerm")
        }
        t.date_issued(:path=>"dateIssued")
        t.year_issued(:path=>"dateOther", :index_as=>[:facetable])
        t.issuance(:index_as=>[:facetable])
      }
      t.note
      t.other_version(:path=>"relatedItem", :attributes=>{:type=>"otherVersion"}) {
        t.location(:path=>"location") {
          t.url(:path=>"url")
        }
      }
      t.peer_reviewed(:proxy=>[:origin_info,:issuance], :index_as=>[:facetable])
    end
  
    # accessor :title, :term=>[:mods, :title_info, :main_title]
    
    # Generates an empty Mods Book (used when you call ModsBook.new without passing in existing xml)
    def self.xml_template
      build_template(:book)
    end    
    
    # Generates a new Person node
    def self.person_template
      build_person(:book)
    end
    
    def to_solr(solr_doc=Solr::Document.new)
      solr_doc = super(solr_doc)
      build_solr(solr_doc)
      solr_doc << {:object_type_facet => HydrangeaBook.solr_object_type_facet }
      solr_doc << Solr::Field.new(:year_i => solr_doc["origin_info_year_issued_facet"]) if solr_doc["origin_info_year_issued_facet"]
      solr_doc
    end
      
  end
end
