module Hydra
    
  class ModsBookPart < ActiveFedora::NokogiriDatastream       

    include Uva::CommonTermMethods

    set_terminology do |t|
      set_basics(t)
      set_sponsor(t)
      
      t.book(:path=>'relatedItem', :attributes=>{:type=>"host"}) {
        t.title_info(:index_as=>[:facetable],:ref=>[:title_info])
        t.editor(:path=>'name'){
          t.first_name(:path=>"namePart", :attributes=>{:type=>"given"})
          t.last_name(:path=>"namePart", :attributes=>{:type=>"family"})
        }
        t.isbn(:path=>"identifier", :attributes=>{:type=>"isbn"})
        t.origin_info(:path=>"originInfo") {
          t.publisher
          t.place {
            t.place_term(:path=>"placeTerm")
          }
          t.date_issued(:path=>"dateIssued")
          t.year_issued(:path=>"dateOther", :index_as=>[:facetable])
          t.issuance(:index_as=>[:facetable])
        }
        t.info(:path=>"part") {
          t.extent
          t.pages(:path=>"extent", :attributes=>{:unit=>"pages"}) {
            t.start
            t.end
          }
          t.start_page(:proxy=>[:pages, :start])
          t.end_page(:proxy=>[:pages, :end])
        }
      }
      t.book_editor_last_name(:proxy=>[:book, :editor, :last_name])
      t.book_editor_first_name(:proxy=>[:book, :editor, :first_name])
      t.note
      t.other_version(:path=>"relatedItem", :attributes=>{:type=>"otherVersion"}) {
        t.location(:path=>"location") {
          t.url(:path=>"url")
        }
      }
      t.peer_reviewed(:proxy=>[:book, :origin_info,:issuance], :index_as=>[:facetable])
    end
  
    # accessor :title, :term=>[:mods, :title_info, :main_title]
    
    # Generates an empty Mods Book Part (used when you call ModsBookPart.new without passing in existing xml)
    def self.xml_template
      build_template(:book_part)
    end    
    
    # Generates a new Person node
    def self.person_template
      build_person(:book_part)
    end
    
    # Generates a new Person node
    def self.editor_template
      build_editor
    end
    
    def to_solr(solr_doc=Solr::Document.new)
      solr_doc = super(solr_doc)
      build_solr(solr_doc)
      solr_doc << {:object_type_facet => HydrangeaBookPart.solr_object_type_facet }
      solr_doc << Solr::Field.new(:year_i => solr_doc["book_origin_info_year_issued_facet"]) if solr_doc["book_origin_info_year_issued_facet"]
      solr_doc
    end
    
    # Remove the contributor entry identified by @contributor_type and @index
    def remove_contributor(contributor_type, index)
      self.find_by_terms(:book, :editor)[index.to_i].remove
      self.dirty = true
    end
      
  end
end
