module Hydra
  
  class ModsConferencePaper < ActiveFedora::NokogiriDatastream       
    
    include Uva::CommonTermMethods
    
    set_terminology do |t|
      set_basics(t)
      set_sponsor(t)

      t.conference_paper(:path=>'relatedItem', :attributes=>{:type=>"host"}) {    
        t.name_ {
          t.conference_name(:path=>'namePart', :attributes=>{:type=>:none})
          t.date(:path=>'namePart', :attributes=>{:type=>'date', :encoding=>"iso8601"})
          t.year(:path=>'namePart', :attributes=>{:type=>'date', :encoding=>:none}, :index_as=>[:facetable])
          t.affiliation
        } 
      }
      t.note
    end
    
    # Generates an empty Mods Conference Paper (used when you call ModsConferencePaper.new without passing in existing xml)
    def self.xml_template
      build_template(:conference_paper)
    end
    
    # Generates a new Person node
    def self.person_template
      build_person(:conference_paper)
    end
    
    def to_solr(solr_doc=Solr::Document.new)
      solr_doc = super(solr_doc)
      build_solr(solr_doc)
      solr_doc << {:object_type_facet => HydrangeaConferencePaper.solr_object_type_facet }        
      solr_doc << Solr::Field.new(:year_i => solr_doc["conference_paper_name_year_facet"]) if solr_doc["conference_paper_name_year_facet"]
      solr_doc
    end
    
  end
end
