module Hydra
    
  class ModsDataset < ActiveFedora::NokogiriDatastream       

    include Uva::CommonTermMethods

    set_terminology do |t|
      set_basics(t)
      t.sponsoring_grant(:path=>"note", :attributes=>{:type=>"funding"})
      t.note(:path=>"note", :attributes=>{:type=>:none})
      t.subject {
        t.topic(:index_as=>[:facetable])
        t.subject_start_date(:path=>"temporal", :attributes=>{:encoding=>"iso8601", :point=>"start"})
        t.subject_end_date(:path=>"temporal", :attributes=>{:encoding=>"iso8601", :point=>"end"})
      }
      t.topic_tag(:proxy=>[:subject, :topic])    
      t.data_collection_start_date(:proxy=>[:subject, :subject_start_date])
      t.data_collection_end_date(:proxy=>[:subject, :subject_end_date])
      t.related_item(:path=>"relatedItem", :attributes=>{:type=>"original"}) {
        t.location {
          t.url
        }
      }
      t.original_location(:proxy=>[:related_item, :location, :url])
      t.referenced_by(:path=>"relatedItem", :attributes=>{:type=>"isReferencedBy"}) {
        t.location {
          t.url
        }
      }
      t.libra_location(:proxy=>[:referenced_by, :location, :url])
      t.origin_info(:path=>"originInfo") {
        t.date_created(:path=>"dateCreated", :attributes=>{:keyDate=>"yes", :encoding=>"iso8601"})
      }
    end
    
    
    # Generates an empty Mods Dataset (used when you call ModsDataset.new without passing in existing xml)
    def self.xml_template
      build_template(:dataset)
    end
    
    # Generates a new Person node
    def self.person_template
      build_person(:dataset)
    end
    
    def to_solr(solr_doc=Solr::Document.new)
      solr_doc = super(solr_doc)
      build_solr(solr_doc)
      solr_doc << {:object_type_facet => HydrangeaDataset.solr_object_type_facet }
      
      year = Date.parse(solr_doc["origin_info_date_issued_t"]).strftime("%Y") rescue ""
      solr_doc << Solr::Field.new(:year_i => year) unless year.blank?
      
      solr_doc
    end
    
    
  end

end