module Hydra
    
  class ModsFourthYearThesis < ActiveFedora::NokogiriDatastream       

    include Uva::CommonTermMethods

    set_terminology do |t|
      set_basics(t)
           
      t.origin_info(:path=>"originInfo") {
        t.date_issued(:path=>"dateIssued")
      }
      t.note
      t.extension_(:path=>"extension") {
        t.degree(:namespace_prefix=>"etd", :path=>"degree") {
          t.name(:namespace_prefix=>"etd", :path=>"name")
          t.level(:namespace_prefix=>"etd", :path=>"level")
          t.discipline(:namespace_prefix=>"etd",:path=>"discipline")
        }
      }
    end
  
    # accessor :title, :term=>[:mods, :title_info, :main_title]
    
    # Generates an empty Mods Fourth Year Thesis (used when you call ModsFourthYearThesis.new without passing in existing xml)
    def self.xml_template
      build_template(:fourth_year_thesis)
    end    
    
    # Generates a new Person node
    def self.person_template
      build_person(:fourth_year_thesis)
    end
    
    def self.advisor_template
      build_advisor
    end
    
    def remove_contributor(contributor_type, index)
      self.find_by_terms('//oxns:name[@type="personal"][contains(oxns:role, "ths")]')[index.to_i].remove
      self.dirty = true
    end
    
    
    def to_solr(solr_doc=Solr::Document.new)
      solr_doc = super(solr_doc)
      build_solr(solr_doc)
      solr_doc << {:object_type_facet => HydrangeaFourthYearThesis.solr_object_type_facet }
      
      year = Date.parse(solr_doc["origin_info_date_issued_t"]).strftime("%Y") rescue ""
      solr_doc << Solr::Field.new(:year_i => year) unless year.blank?
      
      solr_doc
    end
      
  end
end
