module Hydra
  require_dependency 'vendor/plugins/hydra_repository/lib/hydra/rights_metadata.rb'
  class RightsMetadata
    
    set_terminology do |t|
      t.root(:path=>"rightsMetadata", :xmlns=>"http://hydra-collab.stanford.edu/schemas/rightsMetadata/v1", :schema=>"http://github.com/projecthydra/schemas/tree/v1/rightsMetadata.xsd") 
      t.copyright {
        t.machine
        t.human_readable(:path=>"human")
      }
      t.access {
        t.human_readable(:path=>"human")
        t.machine {
          t.group
          t.person
        }
        t.person(:proxy=>[:machine, :person])
        t.group(:proxy=>[:machine, :group])
        # accessor :access_person, :term=>[:access, :machine, :person]
      }
      t.discover_access(:ref=>[:access], :attributes=>{:type=>"discover"})
      t.read_access(:ref=>[:access], :attributes=>{:type=>"read"})
      t.edit_access(:ref=>[:access], :attributes=>{:type=>"edit"}, :index_as=>[:facetable])
      # A bug in OM prevnts us from declaring proxy terms at the root of a Terminology
      # t.access_person(:proxy=>[:access,:machine,:person])
      # t.access_group(:proxy=>[:access,:machine,:group])

      t.embargo {
        t.human_readable(:path=>"human")
        t.machine{
          t.date(:type =>"release")
        }
        t.embargo_release_date(:proxy => [:machine, :date])
      }    
    end
    
    attr_reader :embargo_release_date
    def embargo_release_date=(release_date)
      release_date = release_date.to_s if release_date.is_a? Date
      begin
        Date.parse(release_date)
      rescue 
        return "INVALID DATE"
      end
      self.update_values({[:embargo,:machine,:date]=>release_date})
    end
    def embargo_release_date(opts={})
      embargo_release_date = self.find_by_terms(*[:embargo,:machine,:date]).first ? self.find_by_terms(*[:embargo,:machine,:date]).first.text : nil
      if opts[:format] && opts[:format] == :solr_date
        embargo_release_date << "T23:59:59Z"
      end
      embargo_release_date
    end
    def under_embargo?
      (embargo_release_date && Date.today < embargo_release_date.to_date) ? true : false
    end

    def to_solr(solr_doc=Solr::Document.new)
      super(solr_doc)
      solr_doc << {:embargo_release_date_dt => embargo_release_date(:format=>:solr_date)} if embargo_release_date
      solr_doc
    end

  end
end
