# Constants and other static configuration items unique to UVA.
module Uva::Config

  # URL string constants for UVA
  class URL

    # These constants should be used throughout code rather than hard-coding a
    # URL.  They are included in all compiled *.html.erb templates via a single
    # line of code at the beginning of helpers/application_helper.rb in Rails3:
    #
    #       ActionView::CompiledTemplates.send(:include, UVA::Config)
    #
    # In Rails 2, put following lines instead:
    #
    # def self.included(base)
    #   base.send :include, Uva::Config
    # end
    #
    # Note that any files in the /public directory that contain literal URLs
    # must be maintained separately because they cannot make use of these
    # constants.

    UVA_ROOT              = 'http://www.virginia.edu/'
    UVA                   = UVA_ROOT
    COPYRIGHT             = UVA_ROOT + 'copyright.html'
    HIGHLY_SENSITIVE_DATA = UVA_ROOT + 'informationsecurity/highlysensitivedata/'
    VPR                   = UVA_ROOT + 'vpr/'

    ITS_ROOT              = 'http://its.virginia.edu/'
    ITS                   = ITS_ROOT
    NETBADGE              = ITS_ROOT + 'netbadge/'

    ITC_POLICY_ROOT       = 'https://policy.itc.virginia.edu/'
    SENSITIVE_DATA_POLICY = ITC_POLICY_ROOT + 'policy/policydisplay?id=IRM-015'

    VIRGO_ROOT            = 'http://search.lib.virginia.edu/'
    VIRGO                 = VIRGO_ROOT

    VIRGO_DEV_ROOT        = 'http://searchdev.lib.virginia.edu/'
    VIRGO_DEV             = VIRGO_DEV_ROOT

    IR_ROOT               = 'http://libra.virginia.edu/'
    LIBRA                 = IR_ROOT

    IR_DEV_ROOT           = 'http://irdev.virginia.edu/'
    LIBRA_DEV             = IR_DEV_ROOT

    LIBGUIDES_ROOT        = 'http://guides.lib.virginia.edu/'
    LIBGUIDES             = LIBGUIDES_ROOT
    INTER_LIB_REQUESTS    = LIBGUIDES_ROOT + 'requests/'
    DATABASES             = LIBGUIDES_ROOT + 'az'
    JOURNAL_FINDER        = LIBGUIDES_ROOT + 'journalfinder/'
    FINDING_GOV_INFO      = LIBGUIDES_ROOT + 'findinggovinfo/'

    LIBRARY_ROOT          = 'http://www.library.virginia.edu/'
    LIBRARY               = LIBRARY_ROOT
    HR                    = LIBRARY_ROOT + 'HR/'
    HOURS                 = LIBRARY_ROOT + 'hours/'
    ASKALIBRARIAN         = LIBRARY_ROOT + 'askalibrarian/'
    ASKALIBRARIAN_CONTACT = LIBRARY_ROOT + 'askalibrarian/contact/'
    SERVICES              = LIBRARY_ROOT + 'services/'
    LEO                   = LIBRARY_ROOT + 'services/ils/leo/'
    PURCHASE_REQUESTS     = LIBRARY_ROOT + 'services/purchase-requests/'
    ACCESSIBILITY         = LIBRARY_ROOT + 'services/accessibility-services/'
    COURSE_RESERVES       = LIBRARY_ROOT + 'services/course-reserves/'
    COURSE_RESERVES_PERSONAL = LIBRARY_ROOT + 'services/course-reserves/personal/'
    DIGITIZATION          = LIBRARY_ROOT + 'digitization/#speccol-mats'
    POLICIES              = LIBRARY_ROOT + 'policies/'
    CIRCULATION_POLICY    = LIBRARY_ROOT + 'policies/circulation/'
    SITE_SEARCH           = LIBRARY_ROOT + 'site-search/'
    MAP                   = LIBRARY_ROOT + 'map/'
    PRESS                 = LIBRARY_ROOT + 'press/'
    JOBS                  = LIBRARY_ROOT + 'jobs/'
    STAFF_DIRECTORY       = LIBRARY_ROOT + 'staff/'
    RESEARCH              = LIBRARY_ROOT + 'research/'
    COLLECTIONS           = LIBRARY_ROOT + 'collections/'
    ORGANIZATION          = LIBRARY_ROOT + 'organization/'

    DM_CONSULT_ROOT       = ' http://dmconsult.library.virginia.edu/'
    DM_CONSULT            = DM_CONSULT_ROOT

    LIBRA_FAQ_ROOT        = 'http://library.virginia.edu/libra/'
    LIBRA_FAQ             = LIBRA_FAQ_ROOT
    LIBRA_OA_FAQ          = LIBRA_FAQ_ROOT + 'open-access'
    LIBRA_ETD_FAQ         = LIBRA_FAQ_ROOT + 'etds'
    LIBRA_DATASET_FAQ     = LIBRA_FAQ_ROOT + 'datasets'
    LIBRA_COPYRIGHT_FAQ   = LIBRA_FAQ_ROOT + 'copyright-essentials'
    LIBRA_ETD_SUBMISSION  = LIBRA_FAQ_ROOT + 'etds-checklist'
    LIBRA_DATA            = 'https://dataverse.lib.virginia.edu/'

  end

end
