module Uva::ModsIndexMethods
  # extracts the last_name##full_name##computing_id to be used by home view
  def extract_person_full_names_and_computing_ids
    self.find_by_terms(:person).inject([]) do |names, person|
      name_parts = person.children.inject({}) do |hash,child|
        hash[child.get_attribute(:type)] = child.text if ["family","given"].include? child.get_attribute(:type)
        hash["computing_id"] = child.text if child.name == 'namePart' and child.get_attribute(:type) == nil
        hash
      end
      if name_parts.length == 3 and person.search(:roleTerm).children.text.include?("author")
        if name_parts["family"].blank? && name_parts["given"].blank? && name_parts["computing_id"].blank?
          value = "Unknown Author"
        else
          value = "#{name_parts["family"]}, #{name_parts["given"]} (#{name_parts["computing_id"]})"
        end
        names << Solr::Field.new({"person_full_name_cid_facet".to_sym=>value}) if name_parts.length == 3
      end
      names
    end
  end
  
  # extracts the department for any person who has a computing_id specified  
  def extract_uva_departments
    uva_departments = []
    self.find_by_terms(:person).each do |person|
      name_parts = {}
      person.children.each do |child|
        (name_parts["computing_id"] = child.text) if child.name == "namePart" and child.get_attribute(:type) == nil and !child.text.empty? 
        (name_parts["department"] = child.text) if child.name == "description" and !child.text.empty?
      end
      uva_departments << name_parts["department"] if !name_parts.empty? and name_parts.length == 2
    end
    solr_adds = []
    uva_departments.each do |department|
      solr_adds << Solr::Field.new({"uva_department_facet".to_sym=>department})
      solr_adds << Solr::Field.new({"uva_department_display_facet".to_sym=>uva_department_display_facet(department)})
    end
    solr_adds
  end
  
  def uva_department_display_facet(department)
    mapping = DepartmentFacetMapping.find_by_department_facet(department)
    mapping.nil? ? department : mapping.department_display_facet
  end
  
end
