#!/usr/bin/env ruby -wKU
require 'fastercsv'

module Uva
   class SisTransfer

    attr_accessor :date_string
    attr_accessor :input_data

    def initialize(date_string)
      @date_string = date_string
    end
    
    def load_from_sis
      set_input_data
      write_input_data
    end
    
    def load_to_sis
      write_output_data
    end
   
    def input_file
      SIS_INPUT_DIR + "/UV_Libra_From_SIS_#{@date_string}.txt"
    end
    
    def output_file
      SIS_OUTPUT_DIR + "/UV_LIBRA_IN_#{@date_string}.txt"      
    end
    
    def set_input_data
      @input_data = []
      FasterCSV.foreach(input_file, {:col_sep =>"|", :quote_char => "\001"}) do |row_data|
        @input_data << {
          :empl_id =>  row_data[0],
          :computing_id => row_data[1],
          :first_name => row_data[2],
          :middle_name => row_data[3],
          :last_name => row_data[4],
          :academic_career => row_data[5],
          :academic_program => row_data[6],
          :academic_plan => row_data[7],
          :title_of_work => row_data[8],
          :milestone => row_data[9],
          :plan_degree => row_data[10],
          :date_approved => Date.parse(row_data[11]).strftime('%Y-%m-%d')
        }
      end
    end

    def write_input_data
      count = 0
      @input_data.each do |data|
        record = 
          SisRecord.find_or_create_by_empl_id_and_academic_career_and_academic_program_and_academic_plan_and_milestone(data[:empl_id], data[:academic_career], 
        data[:academic_program], data[:academic_plan], data[:milestone])
        record.computing_id = data[:computing_id]
        record.first_name = data[:first_name]
        record.middle_name = data[:middle_name]
        record.last_name = data[:last_name]
        record.title_of_work = data[:title_of_work]
        record.plan_degree = data[:plan_degree]
        record.date_approved = data[:date_approved]
        record.save
        count += 1
      end
      puts("wrote #{count} records to sis_records table")
    end  
    
    def write_output_data
      records = SisRecord.all(:conditions => ["libra_approved is not null and libra_wrote_to_sis is null and empl_id REGEXP ?", "^[0-9]*$"])
      output = ""
      records.each do |record|
        output += record.empl_id
        output += "|"
        output += record.computing_id
        output += "|"
        output += record.first_name
        output += "|"
        output += record.middle_name
        output += "|"
        output += record.last_name
        output += "|"
        output += record.academic_career
        output += "|"
        output += record.academic_program
        output += "|"
        output += record.academic_plan
        output += "|"
        output += "#{LIBRA_HOST}/catalog/#{record.libra_id}"
        output += "|"
        output += record.milestone
        output += "|"
        output += record.plan_degree
        output += "|"
        output += record.libra_approved.strftime('%m/%d/%Y')
        output += "\n"        
      end
      File.open(output_file, 'w') {|f| f.write(output) }
      records.each do |record|
        record.libra_wrote_to_sis = Date.today
        record.save
      end
      puts("wrote #{records.count} records to #{output_file}")
    end

  end
end