require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')


# See cucumber tests (ie. /features/edit_document.feature) for more tests, including ones that test the edit method & view
# You can run the cucumber tests with 
#
# cucumber --tags @edit
# or
# rake cucumber

describe AssetsController do
  
  before do
    request.env['WEBAUTH_USER']='bob'
  end
  
  it "should use DocumentController" do
    controller.should be_an_instance_of(AssetsController)
  end
  
  describe "update" do
    it "should not allow update if there the user doesn't have editor permissions" do
      mock_user = mock("User")
      mock_user.stubs(:is_being_superuser?).returns(false)
      mock_user.stubs(:can_be_superuser?).returns(false)
      controller.stubs(:current_user).returns(mock_user)
      controller.stubs(:get_solr_response_for_doc_id)
      controller.stubs(:document_editor?).returns(false)
      put :update, {:id=>"_PID_"}
      flash[:error].should == "You don't have sufficient permission to modify this document."
    end
    
    it "should update the object with the attributes provided" do
      mock_document = mock("document")
      mock_document.stubs(:fetch_field_from_params).returns([])
      
      HydrangeaArticle.expects(:find).with("_PID_").returns(mock_document)
      
      desc_metadata_params = {"subject"=>{"0"=>"subject1", "1"=>"subject2", "2"=>"subject3"}, "computing_id"=>{"0"=>"mst3k"}}
      
      simple_request_params = {"asset"=>{
          "descMetadata"=> desc_metadata_params
        }
      }
      
      mock_document.expects(:update_from_computing_id)
      mock_document.expects(:issn_lookup)
      mock_document.expects(:update_from_embargo_selection)
      mock_document.expects(:update_indexed_attributes).with(desc_metadata_params, {:datastreams=>"descMetadata"}).returns({"subject"=>{"2"=>"My Topic"}})
      mock_document.expects(:save)
      controller.stubs(:display_release_status_notice)
      # put :update, :id=>"_PID_", "asset"=>{"subject"=>{"-1"=>"My Topic"}}
      controller.stubs(:verify_editor_permissions).returns(nil)
      put :update, {:id=>"_PID_"}.merge(simple_request_params)
    end
        
    it "should not update the computing id if the item is released for the first person" do
      mock_document = mock("document")
      mock_document.stubs(:is_released?).returns(true)
      mock_document.stubs(:fetch_field_from_params).returns([0, "blah"])
      HydrangeaArticle.expects(:find).with("_PID_").returns(mock_document)
      
      desc_metadata_params = {"person_0_computing_id"=>{"0"=>"asdfdf"}}
      simple_request_params = {"asset"=>{
          "descMetadata"=> desc_metadata_params
        }
      }
      controller.stubs(:verify_editor_permissions).returns(nil)
      put :update, {:id=>"_PID_"}.merge(simple_request_params)
      flash[:error].should == "You may not change the computing id on a released work."
    end
    
    it "should support updating OM::XML datastreams" do
      mock_document = mock("document")
      mock_document.stubs(:fetch_field_from_params).returns([])
      mock_document.stubs(:update_from_computing_id).returns(nil)
      mock_document.stubs(:issn_lookup).returns(nil)
      mock_document.expects(:update_from_embargo_selection).returns(nil)
    
      # content_type is specified as hydrangea_dataset.  If not specified, it defaults to HydrangeaArticle
      HydrangeaDataset.expects(:find).with("_PID_").returns(mock_document)
      
      update_method_args = [ { [{:person=>0}, :role] => {"0"=>"role1","1"=>"role2","2"=>"role3"} }, {:datastreams=>"descMetadata"} ]
      mock_document.expects(:update_indexed_attributes).with( *update_method_args ).returns({"person_0_role"=>{"0"=>"role1","1"=>"role2","2"=>"role3"}})
      mock_document.expects(:save)
      
      
      nokogiri_request_params = {
        "id"=>"_PID_", 
        "content_type"=>"hydrangea_dataset",
        "field_selectors"=>{
          "descMetadata"=>{
            "person_0_role"=>[{":person"=>"0"}, "role"]
          }
        }, 
        "asset"=>{
          "descMetadata"=>{
            "person_0_role"=>{"0"=>"role1", "1"=>"role2", "2"=>"role3"}
          }
        }
      }
      controller.stubs(:display_release_status_notice)
      controller.stubs(:verify_editor_permissions)
      put :update, nokogiri_request_params
      # put :update, :id=>"_PID_", "content_type"=>"hydrangea_article", "datastream"=>"descMetadata", "field_name"=>"person_0_last_name","parent_select"=>[{":person"=>"0"}, ":last_name"], "child_index"=>"0", "value"=>"Sample New Value"
    end
    
    it "should initialize solr" do
      pending
      ActiveFedora::SolrService.expects(:register)
      SaltDocument.expects(:find).with("_PID_").returns(stub_everything)
      put :update, :id=>"_PID_", "asset"=>{"subject"=>{"-1"=>"My Topic"}}
      session[:user].should == 'bob'
    end
  end
  
  describe "new" do
    it "should not allow an etd submission if the person can't upload an etd" do
      mock_user = mock("User")
      mock_user.stubs(:is_being_superuser?).returns(false)
      mock_user.stubs(:can_upload_etd_type?).returns(false)
      controller.stubs(:current_user).returns(mock_user)
      get :new, {:content_type => "hydrangea_doctoral_dissertation"}
      flash[:error].should == "You don't have sufficient permissions to upload that type of document."
    end
    it "should  allow an etd submission if the person can upload an etd" do
      mock_user = mock("User")
      mock_user.stubs(:is_being_superuser?).returns(false)
      mock_user.stubs(:can_upload_etd_type?).returns(true)
      controller.stubs(:current_user).returns(mock_user)
      get :new, {:content_type => "hydrangea_doctoral_dissertation"}
      flash[:error].should be_nil
    end
  end
  
  describe "destroy" do
    it "should delete the asset identified by pid" do
      mock_user = mock("User")
      mock_user.stubs(:is_being_superuser?).returns(true)
      controller.stubs(:current_user).returns(mock_user)
      mock_obj = mock("asset")
      mock_obj.stubs(:delete)
      mock_obj.expects(:datastreams).returns([])
      mock_obj.expects(:destroy_child_assets).returns([])
      ActiveFedora::Base.expects(:load_instance_from_solr).with("__PID__").returns(mock_obj)
      ActiveFedora::ContentModel.expects(:known_models_for).with(mock_obj).returns([HydrangeaArticle])
      HydrangeaArticle.expects(:load_instance_from_solr).with("__PID__").returns(mock_obj)
      controller.stubs(:verify_editor_permissions)
      delete(:destroy, :id => "__PID__")
    end
    it "should not allow destroy if there the user doesn't have editor permissions" do
      mock_user = mock("User")
      mock_user.stubs(:is_being_superuser?).returns(false)
      mock_user.stubs(:can_be_superuser?).returns(false)
      controller.stubs(:current_user).returns(mock_user)
      controller.stubs(:get_solr_response_for_doc_id)
      controller.stubs(:document_editor?).returns(false)
      delete(:destroy, :id => "__PID__")
      flash[:error].should == "You don't have sufficient permission to modify this document."
    end
    
  end
end
