require File.expand_path( File.join( File.dirname(__FILE__), '..','..','spec_helper') )

describe Hydra::ModelMethods do
  it "should set sis values" do
    job = Uva::SisTransfer.new('120211')
    job.stubs('input_file').returns('spec/fixtures/sis_transfer_example.txt')
    job.set_input_data
    job.write_input_data
    dissertation = HydrangeaDoctoralDissertation.new
    dissertation.apply_depositor_metadata("testuser")
    desc_ds = dissertation.datastreams_in_memory["descMetadata"]
    desc_ds.find_by_terms(:person, :computing_id)[0].content.should == "testuser"
    desc_ds.find_by_terms(:person, :first_name)[0].content.should == "Vasudha Chaurey"
    desc_ds.find_by_terms(:person, :last_name)[0].content.should == "Shukla"
    desc_ds.find_by_terms(:person, :institution)[0].content.should == LDAP_INSTITUTION   
    desc_ds.find_by_terms(:person, :description)[0].content.should == "ELECT-PHD"
    desc_ds.find_by_terms(:mods, :title_info, :main_title)[0].content.should == "Electrically Guided Assembly of Nanostructures for Bio-Sensing and Tissue Engineering"
    desc_ds.find_by_terms(:extension, :degree, :level)[0].content.should == "PHD"
    puts desc_ds.find_by_terms(:origin_info, :date_issued)[0].content.should == "2011-10-18"
  end
  
  describe "update from embargo selection" do
    before :each do
      job = Uva::SisTransfer.new('120211')
      job.stubs('input_file').returns('spec/fixtures/sis_transfer_example.txt')
      job.set_input_data
      job.write_input_data
      @dissertation = HydrangeaDoctoralDissertation.new
      @dissertation.apply_depositor_metadata("testuser")
    end
    it "should return if there is no sis data" do
      params = {}
      @dissertation.update_from_embargo_selection(params)
      @dissertation.datastreams["rightsMetadata"].find_by_terms(:embargo, :embargo_release_date).size.should == 0
    end
    it "should update the sis record" do
      params = {:asset=>{:sisMetadata=>{":embargo_selection"=>{"0"=>"24"}}}}
      @dissertation.update_from_embargo_selection(params)
      sis_record = SisRecord.find_by_computing_id_and_milestone_and_libra_approved(@dissertation.depositor, @dissertation.class.sis_type, nil)
      sis_record.embargo_months.should == 24
    end
    it "should update the datastream" do
      params = {:asset=>{:sisMetadata=>{":embargo_selection"=>{"0"=>"24"}}}}
      @dissertation.update_from_embargo_selection(params)
      date_string = (Date.today + 24.months).strftime('%Y-%m-%d')
      @dissertation.datastreams["rightsMetadata"].find_by_terms(:embargo, :embargo_release_date)[0].content.should == date_string
    end
    it "should remove the embargo node if the selection is 0" do
      params = {:asset=>{:sisMetadata=>{":embargo_selection"=>{"0"=>"24"}}}}
      @dissertation.update_from_embargo_selection(params)
      date_string = (Date.today + 24.months).strftime('%Y-%m-%d')
      @dissertation.datastreams["rightsMetadata"].find_by_terms(:embargo, :embargo_release_date)[0].content.should == date_string
      params = {:asset=>{:sisMetadata=>{":embargo_selection"=>{"0"=>"0"}}}}
      @dissertation.update_from_embargo_selection(params)
      date_string = Date.today.strftime('%Y-%m-%d')
      @dissertation.datastreams["rightsMetadata"].find_by_terms(:embargo, :embargo_release_date)[0].content.should == date_string
    end
  end
  
end
