# Need way to find way to stub current_user and RoleMapper in order to run these tests
require File.expand_path( File.join( File.dirname(__FILE__), '..','..','spec_helper') )


describe Uva::SisTransfer do
  describe "initialize" do
    it "should set the date string" do
      job = Uva::SisTransfer.new('120211')
      job.date_string.should == '120211'
    end
  end
  
  describe "input_file" do
    it "should set the filename" do
      job = Uva::SisTransfer.new('120211')
      job.input_file.should == SIS_INPUT_DIR + "/UV_Libra_From_SIS_120211.txt"
    end
  end
  
  describe "set input data" do
    it "should read the file" do
      job = Uva::SisTransfer.new('120211')
      job.stubs('input_file').returns('spec/fixtures/sis_transfer_example.txt')
      job.set_input_data
      job.input_data.size.should == 18
      job.input_data[0][:empl_id].should == '1140948'
      job.input_data[0][:computing_id].should == 'eba4n'
      job.input_data[0][:first_name].should == 'Erik'
      job.input_data[0][:middle_name].should == 'Benjamin'
      job.input_data[0][:last_name].should == 'Alexander'
      job.input_data[0][:academic_career].should == 'GRAD'
      job.input_data[0][:academic_program].should == 'ASG'
      job.input_data[0][:academic_plan].should == 'HIST-PHD'
      job.input_data[0][:title_of_work].should == '"A Revival of the Old Organization": Northern Democrats and Reconstruction, 1868-1876'
      job.input_data[0][:milestone].should == 'PHDDEFENSE'
      job.input_data[0][:plan_degree].should == 'PHD'
      job.input_data[0][:date_approved].should == '2009-11-13'
    end
  end
  
  describe "write data" do
    before :each do
      @job = Uva::SisTransfer.new('120211')
      @job.stubs('input_file').returns('spec/fixtures/sis_transfer_example.txt')
      @job.set_input_data
    end
    it "should write the data" do
      lambda do
        @job.write_input_data
      end.should change(SisRecord, :count).by(18)
    end
    
    it "should not write twice" do
      lambda do
        @job.write_input_data
      end.should change(SisRecord, :count).by(18)
      lambda do
        @job.write_input_data
      end.should change(SisRecord, :count).by(0)      
    end
    
    it "should write the record correctly" do
      @job.write_input_data
      record = SisRecord.find(:first)
      record.empl_id.should == '1140948'
      record.computing_id.should == 'eba4n'
      record.first_name.should == 'Erik'
      record.middle_name.should == 'Benjamin'
      record.last_name.should == 'Alexander'
      record.academic_career.should == 'GRAD'
      record.academic_program.should == 'ASG'
      record.academic_plan.should == 'HIST-PHD'
      record.title_of_work.should == '"A Revival of the Old Organization": Northern Democrats and Reconstruction, 1868-1876'
      record.milestone.should == 'PHDDEFENSE'
      record.plan_degree.should == 'PHD'
      record.date_approved.strftime('%Y-%m-%d').should == '2009-11-13'
      record.libra_approved.should be_nil
      record.libra_id.should be_nil
    end
  end
  
end