require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')
require "active_fedora"
require "nokogiri"

describe HydrangeaArticlePreprint do
  
  before(:each) do
    Fedora::Repository.stubs(:instance).returns(stub_everything())
    @article = HydrangeaArticlePreprint.new
    @article.stubs(:apply_ldap_values)
    @file_asset = FileAsset.new
  end
  
  describe "test release readiness" do
    it "should not release until a file is uploaded and author, title, release_to, and journal title are set" do
      @article.datastreams["properties"].released_values = "true"
      @article.to_solr.should_not have_solr_fields(:read_access_group_t=>"public")
      @article.datastreams["descMetadata"].update_indexed_attributes(
       {[:title_info, :main_title]=>"my title",
        [:journal, :title_info, :main_title]=>"my journal title", 
        [{:person=>0}, :last_name]=>"author_last_name", 
        [{:person=>0}, :role, :text]=>"author"})
      @article.datastreams["properties"].update_indexed_attributes({[:release_to]=>"public"})
      @article.stubs(:parts).returns([@file_asset])
      @article.to_solr.should_not have_solr_fields(:read_access_group_t=>"public")
      # adding for UVa Libra implementation
      @article.datastreams["properties"].release_to_values = "public"
      @article.to_solr.should have_solr_fields(:read_access_group_t=>"public")
    end
  end
  
end