require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')
require "active_fedora"
require "nokogiri"

describe HydrangeaBookChapter do
  
  before(:each) do
    Fedora::Repository.stubs(:instance).returns(stub_everything())
    @chapter = HydrangeaBookChapter.new
    @chapter.stubs(:apply_ldap_values)
    @file_asset = FileAsset.new
  end
  
  describe "test release readiness" do
    it "should not release until a file is uploaded and author, title, release_to, book title and publisher name are set" do
      @chapter.datastreams["properties"].released_values = "true"
      @chapter.to_solr.should_not have_solr_fields(:read_access_group_t=>"public")
      @chapter.datastreams["descMetadata"].update_indexed_attributes(
       {[:title_info, :main_title]=>"my title",
        [{:person=>0}, :last_name]=>"author_last_name", 
        [{:person=>0}, :role, :text]=>"author",
        [:book, :title_info, :main_title]=>"book title",
        [:book, :origin_info, :publisher]=>"publisher"}
        )
      @chapter.datastreams["properties"].update_indexed_attributes({[:release_to]=>"public"})
      @chapter.stubs(:parts).returns([@file_asset])
      @chapter.to_solr.should_not have_solr_fields(:read_access_group_t=>"public")
      # adding for UVa Libra implementation
      @chapter.datastreams["properties"].release_to_values = "public"
      @chapter.to_solr.should have_solr_fields(:read_access_group_t=>"public")
    end
  end
  
end