require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')
require "active_fedora"
require "nokogiri"

describe HydrangeaBook do
  
  before(:each) do
    Fedora::Repository.stubs(:instance).returns(stub_everything())
    @book = HydrangeaBook.new
    @book.stubs(:apply_ldap_values)
    @file_asset = FileAsset.new
  end
  
  describe "test release readiness" do
    it "should not release until author, title, publisher, release_to, and doc have been set" do
      @book.datastreams["properties"].released_values = "true"
      @book.to_solr.should_not have_solr_fields(:read_access_group_t=>"public")
      @book.datastreams["descMetadata"].update_indexed_attributes(
      {[:title_info, :main_title]=>"my title", 
        [:origin_info, :publisher]=>"my publisher", 
        [{:person=>0}, :last_name]=>"author_last_name", 
        [{:person=>0}, :role, :text]=>"author"})
      @book.to_solr.should_not have_solr_fields(:read_access_group_t=>"public")
      @book.stubs(:parts).returns([@file_asset])
      # adding for UVa Libra implementation
      @book.datastreams["properties"].release_to_values = "public"
      @book.to_solr.should have_solr_fields(:read_access_group_t=>"public")
    end
  end
  
end