require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')
require "active_fedora"
require "nokogiri"

describe HydrangeaConferencePaper do
  
  before(:each) do
    Fedora::Repository.stubs(:instance).returns(stub_everything())
    @paper = HydrangeaConferencePaper.new
    @paper.stubs(:apply_ldap_values)
    @file_asset = FileAsset.new
  end
  
  describe "test release readiness" do
    it "should not release until a file is uploaded and author, title, release_to, and conference name are set" do
      @paper.datastreams["properties"].released_values = "true"
      @paper.to_solr.should_not have_solr_fields(:read_access_group_t=>"public")
      @paper.datastreams["descMetadata"].update_indexed_attributes(
       {[:title_info, :main_title]=>"my title",
        [:conference_paper, :name, :conference_name]=>"my conference", 
        [{:person=>0}, :last_name]=>"author_last_name", 
        [{:person=>0}, :role, :text]=>"author"})
      @paper.datastreams["properties"].update_indexed_attributes({[:release_to]=>"public"})
      @paper.stubs(:parts).returns([@file_asset])
      @paper.to_solr.should_not have_solr_fields(:read_access_group_t=>"public")
      # adding for UVa Libra implementation
      @paper.datastreams["properties"].release_to_values = "public"
      @paper.to_solr.should have_solr_fields(:read_access_group_t=>"public")
    end
  end
  
end