require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')
require "active_fedora"
require "nokogiri"

describe HydrangeaDoctoralDissertation do
  
  before(:each) do
    Fedora::Repository.stubs(:instance).returns(stub_everything())
    @dissertation = HydrangeaDoctoralDissertation.new
    @dissertation.stubs(:apply_ldap_values)
    @file_asset = FileAsset.new
  end
  
  describe "test release readiness" do
    it "should not release until a file is uploaded and title, author, advisor, approved, and release_to are set" do
      @dissertation.datastreams["properties"].released_values = "true"
      @dissertation.to_solr.should_not have_solr_fields(:read_access_group_t=>"uetd")
      @dissertation.datastreams["descMetadata"].update_indexed_attributes(
       {[:title_info, :main_title]=>"my title",
        [{:person=>0}, :last_name]=>"author_last_name", 
        [{:person=>0}, :role, :text]=>"author",
        [{:person=>1}, :last_name]=>"advisor_last_name", 
        [{:person=>1}, :role, :text]=>"advisor",
        [:origin_info, :date_issued]=>"20120212"
        }
        )
      @dissertation.datastreams["properties"].update_indexed_attributes({[:release_to]=>"uetd"})
      @dissertation.stubs(:parts).returns([@file_asset])
      @dissertation.to_solr.should_not have_solr_fields(:read_access_group_t=>"uetd")
      # adding for UVa Libra implementation
      @dissertation.datastreams["properties"].release_to_values = "uetd"
      @dissertation.to_solr.should have_solr_fields(:release_to_t=>"uetd")
    end
  end
  
  describe "sis_type" do
    it "should return the sis type" do
      HydrangeaDoctoralDissertation.sis_type.should == 'PHDDEFENSE'
    end
  end
  
  describe "libra_type" do
    it "should return the libra type" do
      HydrangeaDoctoralDissertation.libra_type.should == 'hydrangea_doctoral_dissertation'
    end
  end
  
end
