require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')
require "active_fedora"
require "nokogiri"

describe HydrangeaFourthYearThesis do
  
  before(:each) do
    Fedora::Repository.stubs(:instance).returns(stub_everything())
    @thesis = HydrangeaFourthYearThesis.new
    @thesis.stubs(:apply_ldap_values)
    @file_asset = FileAsset.new
  end
  
  describe "test release readiness" do
    it "should not release until a file is uploaded and title, author, advisor, approved, and release_to are set" do
      @thesis.datastreams["properties"].released_values = "true"
      @thesis.to_solr.should_not have_solr_fields(:read_access_group_t=>"uetd")
      @thesis.datastreams["descMetadata"].update_indexed_attributes(
       {[:title_info, :main_title]=>"my title",
        [{:person=>0}, :last_name]=>"author_last_name", 
        [{:person=>0}, :role, :text]=>"author",
        [{:person=>1}, :last_name]=>"advisor_last_name", 
        [{:person=>1}, :role, :text]=>"advisor",
        [:origin_info, :date_issued]=>"20120212"
        }
        )
      @thesis.datastreams["properties"].update_indexed_attributes({[:release_to]=>"uetd"})
      @thesis.stubs(:parts).returns([@file_asset])
      @thesis.to_solr.should_not have_solr_fields(:read_access_group_t=>"uetd")
      # adding for UVa Libra implementation
      @thesis.datastreams["properties"].release_to_values = "uetd"
      @thesis.to_solr.should have_solr_fields(:release_to_t =>"uetd")
    end
  end
  
  describe "sis_type" do
    it "should return the sis type" do
      HydrangeaFourthYearThesis.sis_type.should == 'NOT_DEFINED_YET'
    end
  end
  
  describe "libra_type" do
    it "should return the libra type" do
      HydrangeaFourthYearThesis.libra_type.should ==  'hydrangea_fourth_year_thesis'
    end
  end
  
end
