# Be sure to restart your server when you modify this file

# Uncomment below to force Rails into production mode when
# you don't control web/app server and can't set it the proper way
# ENV['RAILS_ENV'] ||= 'production'

# Specifies gem version of Rails to use when vendor/rails is not present
RAILS_GEM_VERSION = '2.3.5' unless defined? RAILS_GEM_VERSION

# Bootstrap the Rails environment, frameworks, and default configuration
require File.join(File.dirname(__FILE__), 'boot')

# we need this so asset helpers work as expected: <%= stylesheet_link_tag 'blah', :plugin=>'blacklight' %>
require File.join(File.dirname(__FILE__), '../vendor/plugins/engines/boot')

Rails::Initializer.run do |config|
  # Settings in config/environments/* take precedence over those specified here.
  # Application configuration should go into files in config/initializers
  # -- all .rb files in that directory are automatically loaded.
  # See Rails::Configuration for more options.
  config.action_mailer.delivery_method = :sendmail

  # Skip frameworks you're not going to use (only works if using vendor/rails).
  # To use Rails without a database, you must remove the Active Record framework
  #config.frameworks -= [ :action_mailer ]
  
  # Only load the plugins named here, in the order given. By default, all plugins 
  # in vendor/plugins are loaded in alphabetical order.
  # :all can be used as a placeholder for all plugins not explicitly named
  #config.plugins = %W(acts_as_taggable_on_steroids simplest_auth resource_controller)
  #config.plugins = %W(acts_as_taggable_on_steroids resource_controller)
  
  # Add additional load paths for your own custom dirs
  # config.load_paths += %W( #{RAILS_ROOT}/extras )
  
  # Force all environments to use the same logger level
  # (by default production uses :info, the others :debug)
  config.log_level = :debug
  
  # Make Time.zone default to the specified zone, and make Active Record store time values
  # in the database in UTC, and return them converted to the specified local zone.
  # Run "rake -D time" for a list of tasks for finding time zone names. Comment line to use default local time.
  config.time_zone = 'UTC'
  
  # Your secret key for verifying cookie session data integrity.
  # If you change this key, all old sessions will become invalid!
  # Make sure the secret is at least 30 characters and all random, 
  # no regular words or you'll be exposed to dictionary attacks.
  config.action_controller.session = {
    :session_key => '_blacklight_session',
    :secret      => '3e37cf3b7a9a3359f437aac207241fd25c2e2a107f85b2e6d32e0b5e3795e75fdb094b9d045d8c40e9ae2b38063c8926ef01b1e03946652eadf96c653d6effa9'
  }
  
  # Use the database for sessions instead of the cookie-based default,
  # which shouldn't be used to store highly confidential information
  # (create the session table with 'rake db:sessions:create')
  # config.action_controller.session_store = :active_record_store

  # Use SQL instead of Active Record's schema dumper when creating the test database.
  # This is necessary if your schema can't be completely dumped by the schema dumper,
  # like if you have constraints or database-specific column types
  # config.active_record.schema_format = :sql

  # Activate observers that should always be running
  # config.active_record.observers = :cacher, :garbage_collector
  
  # Make Active Record use UTC-base instead of local time
  # config.active_record.default_timezone = :utc
  
  #
  # list test dependencies here...
  #

  #unicode is used by refworks export to force C form normalization, which
  #refworks wants.
  config.gem 'unicode'
  
  # these are used for testing (rake solr:spec, which ends up running in the "development" environment)
  config.gem 'rspec', :version=>'1.3.0', :lib=>false
  config.gem 'rspec-rails', :version=>'1.3.2', :lib=>false
  
  config.gem 'webrat', :version=>'0.7.0'
  config.gem 'cucumber', :version=>'0.6.2'
  config.gem 'rcov', :version=>'0.9.7.1'
  
  
  # these are auto-generated by cucumber and usually in the environments/cucumber.rb file
  # -- but having them here makes it easy to do rake gems:install without a RAILS_ENV=cucumber
  config.gem 'cucumber-rails',   :lib => false, :version => '>=0.2.4'
  config.gem 'database_cleaner', :lib => false, :version => '>=0.4.3'
  config.gem 'webrat',           :lib => false, :version => '>=0.6.0'
  config.gem 'rspec',            :lib => false, :version => '>=1.3.0'
  config.gem 'rspec-rails',      :lib => false, :version => '>=1.3.2'
  
  # rdoc and haml are required by hana
  config.gem 'rdoc', :version=>'2.4.3'
  config.gem 'haml', :version=>'2.2.19'
  
  # hanna is only needed for generating rdocs
  config.gem 'hanna', :lib=>'hanna/rdoctask', :source=>'http://gemcutter.org', :version=>'0.1.12'
  
  # got this idea from Rails' Initializer#load_environment()
  # it's reading the file and evaluating it in the context of this code
  # This way, we keep things dry between Blacklight's stand-alone testing and 
  # running as a plugin.
  init_path = File.join(File.dirname(__FILE__), '..', 'init.rb')
  eval(IO.read(init_path), binding, init_path)
  
end