require 'mediashelf/active_fedora_helper'
class CatalogController
  
  include Blacklight::CatalogHelper
  include Hydra::RepositoryController
  include Hydra::AccessControlsEnforcement
  before_filter :require_solr, :require_fedora, :only=>[:show, :edit, :index]
  before_filter :set_solr_document, :only=>[:edit, :show]
  before_filter :set_fedora_document, :only=>[:edit, :show]
  before_filter :check_show_permission, :only=>:show
  before_filter :check_edit_permission, :only=>:edit
  
  def edit
    session[:viewing_context] = "edit"
    render :action=>:show
  end

  # displays values and pagination links for a single facet field
  def facet
    # adding the following for facet_pagination with Lucene queries to avoide NPE
    params[:qt] = "dismax"
    @pagination = get_facet_pagination(params[:id], params)
  end
  
  # get search results from the solr index
  def index
    @extra_controller_params ||= {}
    (@response, @document_list) = get_search_results( @extra_controller_params.merge!(:q=>build_lucene_query(params[:q])) )
    logger.debug("LUCENE QUERY: #{build_lucene_query(params[:q])}")
    logger.debug("FOUND: #{@document_list.length}")
    logger.debug("RESPONSE: #{@response.inspect}")
    logger.debug("DOCUMENT: #{@document_list.inspect}")
    @filters = params[:f] || []
    respond_to do |format|
      format.html { save_current_search_params }
      format.rss  { render :layout => false }
    end
    rescue RSolr::RequestError
      logger.error("Unparseable search error: #{params.inspect}" ) 
      flash[:notice] = "Sorry, I don't understand your search." 
      redirect_to :action => 'index', :q => nil , :f => nil
    rescue 
      logger.error("Unknown error: #{params.inspect}" ) 
      flash[:notice] = "Sorry, you've encountered an error. Try a different search." 
      redirect_to :action => 'index', :q => nil , :f => nil
  end
    
  def show
    if params[:viewing_context] == "browse"
      session[:viewing_context] = params[:viewing_context]
    elsif session[:viewing_context] == "edit"
      if editor?
        if params[:files]
          redirect_to(:action=>:edit, :files=>true) and return
        else
          redirect_to(:action=>:edit) and return
        end
      else
        # the person isn't an editor, switch them back to browse
        session[:viewing_context] = "browse"
      end
    end
    params = {:qt=>"dismax",:q=>"*:*",:rows=>"0",:facet=>"true", :facets=>{:fields=>Blacklight.config[:facet][:field_names]}}
    @facet_lookup = Blacklight.solr.find params
  end

  # 
  ### This was how get_search_results in SALT deals with switching solr instances
  #
  # def get_search_results(extra_controller_params={})
  #   _search_params = self.solr_search_params(extra_controller_params)
  #   index = _search_params[:qt] == 'fulltext' ? :fulltext : :default
  #   
  #   document_list = solr_response.docs.collect {|doc| SolrDocument.new(doc)}
  #   
  #   Blacklight.solr(index).find(_search_params)
  #   
  #   return [solr_response, document_list]
  #   
  # end
  protected
  
  # a solr query method
  # this is used when selecting a search result: we have a query and a 
  # position in the search results and possibly some facets
  def get_single_doc_via_search(extra_controller_params={})
    solr_params = solr_search_params(extra_controller_params)
    solr_params[:per_page] = 1
    solr_params[:fl] = '*'
    if params[:q].to_s.blank?
      solr_params.merge!(:q=>build_lucene_query(params[:q]))
    end
    Blacklight.solr.find(solr_params).docs.first
  end

  # This method will remove certain params from the session[:search] hash
  # if the values are blank? (nil or empty string)
  # if the values aren't blank, they are saved to the session in the :search hash.
  # We're overriding this for SALT because we need to add in the view parameter to 
  # make sure that the user is taken back to the same view (gallery/list) that they came from
  def delete_or_assign_search_session_params
    [:q, :qt, :search_field, :f, :per_page, :page, :sort, :view].each do |pname|
      params[pname].blank? ? session[:search].delete(pname) : session[:search][pname] = params[pname]
    end
  end
  
  def setup_next_document
    @next_document = (session[:search][:counter] && session[:search][:counter].to_i > 1) ? setup_document_by_counter(session[:search][:counter].to_i + 1) : nil
  end
  
  # loads up the document from fedora
  def set_fedora_document
    af_base = ActiveFedora::Base.load_instance(params[:id])
    the_model = ActiveFedora::ContentModel.known_models_for( af_base ).first
    if the_model.nil?
      the_model = DcDocument
    end
    @document_fedora = the_model.load_instance(params[:id])
  end
  
  # loads up the document from solr
  def set_solr_document
    @response, @document = get_solr_response_for_doc_id
  end
  
  # ensures that the document has been released
  def check_show_permission
    return if reader?
    unless @document['released_facet'] && @document['released_facet'].first == "true"
      flash[:notice]= "You do not have sufficient access privileges to read this document, which has been marked private."
      redirect_to(:action => :index, :q => nil , :f => nil) and return
    end
   end

  # ensures that the document is editable
  def check_edit_permission
    if !editor?
      session[:viewing_context] = "browse"
      flash[:notice] = "You do not have sufficient privileges to edit this document. You have been redirected to the read-only view."
      redirect_to :action=>:show
    end
  end
end
